# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import TestCase, mock
from delfin import exception
from delfin import context
from delfin.drivers.huawei.oceanstor.oceanstor import OceanStorDriver, consts
from delfin.drivers.huawei.oceanstor.rest_client import RestClient
from requests import Session


class Request:
    def __init__(self):
        self.environ = {'delfin.context': context.RequestContext()}
        pass


ACCESS_INFO = {
    "storage_id": "12345",
    "vendor": "dell_emc",
    "model": "vmax",
    "rest": {
        "host": "10.0.0.1",
        "port": "8443",
        "username": "user",
        "password": "pass",
    },
    "extra_attributes": {
        "array_id": "00112233"
    }
}


def create_driver():
    kwargs = ACCESS_INFO

    m = mock.MagicMock()
    with mock.patch.object(Session, 'post', return_value=m):
        m.raise_for_status.return_value = None
        m.json.return_value = {
            'data': {
                'deviceid': '123ABC456',
                'iBaseToken': 'FFFF0000',
                'accountstate': 1
            },
            'error': {
                'code': 0,
                'description': '0'
            }
        }
        return OceanStorDriver(**kwargs)


class TestOceanStorStorageDriver(TestCase):

    def test_init(self):
        driver = create_driver()
        self.assertEqual(driver.storage_id, "12345")
        self.assertEqual(driver.sector_size, consts.SECTORS_SIZE)
        self.assertEqual(driver.client.device_id, '123ABC456')

        m = mock.MagicMock()
        with mock.patch.object(Session, 'post', return_value=m):
            m.raise_for_status.return_value = None
            m.json.return_value = {
                'data': {
                    'deviceid': '123ABC456',
                    'iBaseToken': 'FFFF0000',
                    'accountstate': 1
                },
                'error': {
                    'code': 123,
                    'description': '0'
                }
            }
            kwargs = ACCESS_INFO
            with self.assertRaises(Exception) as exc:
                OceanStorDriver(**kwargs)
            self.assertIn('The credentials are invalid', str(exc.exception))

    def test_get_storage(self):
        driver = create_driver()
        expected = {
            'name': 'OceanStor',
            'vendor': 'Huawei',
            'description': 'Huawei OceanStor Storage',
            'model': 'OceanStor_1',
            'status': 'normal',
            'serial_number': '012345',
            'firmware_version': '1000',
            'location': 'Location1',
            'total_capacity': 51200,
            'used_capacity': 38400,
            'free_capacity': 20480,
            'raw_capacity': 76800
        }

        ret = [
            # Storage 1
            {
                'data': {
                    'RUNNINGSTATUS': '1',
                    'SECTORSIZE': '512',
                    'TOTALCAPACITY': '100',
                    'USEDCAPACITY': '75',
                    'MEMBERDISKSCAPACITY': '150',
                    'userFreeCapacity': '40',
                    'NAME': 'OceanStor_1',
                    'ID': '012345',
                    'LOCATION': 'Location1'
                },
                'error': {
                    'code': 0,
                    'description': '0'
                }
            },
            {
                'data': [{
                    'SOFTVER': '1000',
                }],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            }
        ]
        with mock.patch.object(RestClient, 'do_call', side_effect=ret):
            storage = driver.get_storage(context)
            self.assertDictEqual(storage, expected)

    def test_list_storage_pools(self):
        driver = create_driver()
        expected = [
            {
                'name': 'OceanStor_1',
                'storage_id': '12345',
                'native_storage_pool_id': '012345',
                'description': 'Huawei OceanStor Pool',
                'status': 'normal',
                'storage_type': 'block',
                'total_capacity': 51200,
                'used_capacity': 38400,
                'free_capacity': 20480
            },
            {
                'name': 'OceanStor_1',
                'storage_id': '12345',
                'native_storage_pool_id': '012345',
                'description': 'Huawei OceanStor Pool',
                'status': 'offline',
                'storage_type': 'file',
                'total_capacity': 51200,
                'used_capacity': 38400,
                'free_capacity': 20480
            }

        ]

        ret = [
            {
                'data': [
                    {
                        'RUNNINGSTATUS': '27',
                        'USAGETYPE': '1',
                        'USERTOTALCAPACITY': '100',
                        'USERCONSUMEDCAPACITY': '75',
                        'USERFREECAPACITY': '40',
                        'NAME': 'OceanStor_1',
                        'ID': '012345',
                        'LOCATION': 'Location1'
                    },
                    {
                        'RUNNINGSTATUS': '28',
                        'USAGETYPE': '2',
                        'USERTOTALCAPACITY': '100',
                        'USERCONSUMEDCAPACITY': '75',
                        'USERFREECAPACITY': '40',
                        'NAME': 'OceanStor_1',
                        'ID': '012345',
                        'LOCATION': 'Location1'
                    }
                ],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            },
            {
                'data': [{
                    'SOFTVER': '1000',
                }],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            }
        ]
        with mock.patch.object(RestClient, 'do_call', side_effect=ret):
            pools = driver.list_storage_pools(context)
            self.assertDictEqual(pools[0], expected[0])
            self.assertDictEqual(pools[1], expected[1])

        with mock.patch.object(RestClient, 'get_all_pools',
                               side_effect=exception.DelfinException):
            with self.assertRaises(Exception) as exc:
                driver.list_storage_pools(context)
            self.assertIn('Exception from Storage Backend',
                          str(exc.exception))

    def test_list_volumes(self):
        driver = create_driver()
        expected = [
            {
                'name': 'Volume_1',
                'storage_id': '12345',
                'description': 'Huawei OceanStor volume',
                'status': 'available',
                'native_volume_id': '0001',
                'native_storage_pool_id': '012345',
                'wwn': 'wwn12345',
                'type': 'thin',
                'total_capacity': 51200,
                'used_capacity': 38400,
                'free_capacity': None,
                'compressed': False,
                'deduplicated': False
            },
            {
                'name': 'Volume_1',
                'storage_id': '12345',
                'description': 'Huawei OceanStor volume',
                'status': 'error',
                'native_volume_id': '0001',
                'native_storage_pool_id': '012345',
                'wwn': 'wwn12345',
                'type': 'thick',
                'total_capacity': 51200,
                'used_capacity': 38400,
                'free_capacity': None,
                'compressed': True,
                'deduplicated': True
            }

        ]

        ret = [
            {
                'data': [
                    {
                        'RUNNINGSTATUS': '27',
                        'USAGETYPE': '1',
                        'CAPACITY': '100',
                        'ALLOCCAPACITY': '75',
                        'WWN': 'wwn12345',
                        'NAME': 'Volume_1',
                        'ID': '0001',
                        'LOCATION': 'Location1',
                        'PARENTNAME': 'OceanStor_1',
                        'ENABLECOMPRESSION': 'false',
                        'ENABLEDEDUP': 'false',
                        'ALLOCTYPE': '1',
                        'SECTORSIZE': '512',

                    },
                    {
                        'RUNNINGSTATUS': '28',
                        'USAGETYPE': '1',
                        'CAPACITY': '100',
                        'ALLOCCAPACITY': '75',
                        'WWN': 'wwn12345',
                        'NAME': 'Volume_1',
                        'ID': '0001',
                        'LOCATION': 'Location1',
                        'PARENTNAME': 'OceanStor_1',
                        'ENABLECOMPRESSION': 'true',
                        'ENABLEDEDUP': 'true',
                        'ALLOCTYPE': '0',
                        'SECTORSIZE': '512',

                    }
                ],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            },
            {
                'data': [{
                    'NAME': 'OceanStor_1',
                    'ID': '012345'
                }],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            },
            {
                'data': [{
                    'SOFTVER': '1000',
                }],
                'error': {
                    'code': 0,
                    'description': '0'
                }
            }
        ]
        with mock.patch.object(RestClient, 'do_call', side_effect=ret):
            volumes = driver.list_volumes(context)
            self.assertDictEqual(volumes[0], expected[0])
            self.assertDictEqual(volumes[1], expected[1])

        with mock.patch.object(RestClient, 'get_all_volumes',
                               side_effect=exception.DelfinException):
            with self.assertRaises(Exception) as exc:
                driver.list_volumes(context)
            self.assertIn('Exception from Storage Backend',
                          str(exc.exception))



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from delfin import db
from delfin.api import validation
from delfin.api.common import wsgi
from delfin.api.schemas import access_info as schema_access_info
from delfin.api.views import access_info as access_info_viewer
from delfin.common import constants
from delfin.drivers import api as driverapi


class AccessInfoController(wsgi.Controller):

    def __init__(self):
        super(AccessInfoController, self).__init__()
        self._view_builder = access_info_viewer.ViewBuilder()
        self.driver_api = driverapi.API()

    def show(self, req, id):
        """Show access information by storage id."""
        ctxt = req.environ['delfin.context']
        access_info = db.access_info_get(ctxt, id)
        return self._view_builder.show(access_info)

    @validation.schema(schema_access_info.update)
    def update(self, req, id, body):
        """Update storage access information."""
        ctxt = req.environ.get('delfin.context')
        access_info = db.access_info_get(ctxt, id)
        for access in constants.ACCESS_TYPE:
            if not body.get(access):
                body[access] = None
        access_info.update(body)
        access_info = self.driver_api.update_access_info(ctxt, access_info)

        return self._view_builder.show(access_info)


def create_resource():
    return wsgi.Resource(AccessInfoController())



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import requests
import six
import urllib3
from urllib3.exceptions import InsecureRequestWarning
from oslo_log import log as logging

from delfin import exception
from delfin.drivers.huawei.oceanstor import consts
from delfin.ssl_utils import HostNameIgnoreAdapter
from delfin.i18n import _

LOG = logging.getLogger(__name__)


class RestClient(object):
    """Common class for Huawei OceanStor storage system."""

    def __init__(self, **kwargs):

        rest_access = kwargs.get('rest')
        if rest_access is None:
            raise exception.InvalidInput('Input rest_access is missing')
        self.rest_host = rest_access.get('host')
        self.rest_port = rest_access.get('port')
        self.rest_username = rest_access.get('username')
        self.rest_password = rest_access.get('password')

        # Lists of addresses to try, for authorization
        address = 'https://%(host)s:%(port)s/deviceManager/rest/' % \
                  {'host': self.rest_host, 'port': str(self.rest_port)}
        self.san_address = [address]
        self.session = None
        self.url = None
        self.device_id = None
        self.verify = None
        urllib3.disable_warnings(InsecureRequestWarning)
        self.reset_connection(**kwargs)

    def reset_connection(self, **kwargs):
        self.verify = kwargs.get('verify', False)
        try:
            self.login()
        except Exception as ex:
            msg = "Failed to login to OceanStor: {}".format(ex)
            LOG.error(msg)
            raise exception.InvalidCredential(msg)

    def init_http_head(self):
        self.url = None
        self.session = requests.Session()
        self.session.headers.update({
            "Connection": "keep-alive",
            "Content-Type": "application/json"})
        if not self.verify:
            self.session.verify = False
        else:
            LOG.debug("Enable certificate verification, verify: {0}".format(
                self.verify))
            self.session.verify = self.verify
            self.session.mount("https://", HostNameIgnoreAdapter())

        self.session.trust_env = False

    def do_call(self, url, data, method,
                calltimeout=consts.SOCKET_TIMEOUT, log_filter_flag=False):
        """Send requests to Huawei storage server.

        Send HTTPS call, get response in JSON.
        Convert response into Python Object and return it.
        """
        if self.url:
            url = self.url + url

        kwargs = {'timeout': calltimeout}
        if data:
            kwargs['data'] = json.dumps(data)

        if method in ('POST', 'PUT', 'GET', 'DELETE'):
            func = getattr(self.session, method.lower())
        else:
            msg = _("Request method %s is invalid.") % method
            LOG.error(msg)
            raise exception.StorageBackendException(reason=msg)

        try:
            res = func(url, **kwargs)
        except requests.exceptions.SSLError as ssl_exc:
            LOG.exception('SSLError exception from server: %(url)s.'
                          ' Error: %(err)s', {'url': url, 'err': ssl_exc})
            return {"error": {"code": consts.ERROR_CONNECT_TO_SERVER,
                              "description": "Retry with valid certificate."}}
        except Exception as err:
            LOG.exception('Bad response from server: %(url)s.'
                          ' Error: %(err)s', {'url': url, 'err': err})
            return {"error": {"code": consts.ERROR_CONNECT_TO_SERVER,
                              "description": "Connect to server error."}}

        try:
            res.raise_for_status()
        except requests.HTTPError as exc:
            return {"error": {"code": exc.response.status_code,
                              "description": six.text_type(exc)}}

        res_json = res.json()
        if not log_filter_flag:
            LOG.info('\n\n\n\nRequest URL: %(url)s\n\n'
                     'Call Method: %(method)s\n\n'
                     'Request Data: %(data)s\n\n'
                     'Response Data:%(res)s\n\n',
                     {'url': url,
                      'method': method,
                      'data': data,
                      'res': res_json})

        return res_json

    def login(self):
        """Login Huawei storage array."""
        device_id = None
        for item_url in self.san_address:
            url = item_url + "xx/sessions"
            data = {"username": self.rest_username,
                    "password": self.rest_password,
                    "scope": "0"}
            self.init_http_head()
            result = self.do_call(url, data, 'POST',
                                  calltimeout=consts.LOGIN_SOCKET_TIMEOUT,
                                  log_filter_flag=True)

            if (result['error']['code'] != 0) or ("data" not in result):
                LOG.error("Login error. URL: %(url)s\n"
                          "Reason: %(reason)s.",
                          {"url": item_url, "reason": result})
                continue

            LOG.debug('Login success: %(url)s', {'url': item_url})
            device_id = result['data']['deviceid']
            self.device_id = device_id
            self.url = item_url + device_id
            self.session.headers['iBaseToken'] = result['data']['iBaseToken']
            if (result['data']['accountstate']
                    in (consts.PWD_EXPIRED, consts.PWD_RESET)):
                self.logout()
                msg = _("Password has expired or has been reset, "
                        "please change the password.")
                LOG.error(msg)
                raise exception.StorageBackendException(reason=msg)
            break

        if device_id is None:
            msg = _("Failed to login with all rest URLs.")
            LOG.error(msg)
            raise exception.StorageBackendException(reason=msg)

        return device_id

    def call(self, url, data=None, method=None, log_filter_flag=False):
        """Send requests to server.

        If fail, try another RestURL.
        """
        device_id = None
        old_url = self.url
        result = self.do_call(url, data, method,
                              log_filter_flag=log_filter_flag)
        error_code = result['error']['code']
        if (error_code == consts.ERROR_CONNECT_TO_SERVER
                or error_code == consts.ERROR_UNAUTHORIZED_TO_SERVER):
            LOG.error("Can't open the recent url, relogin.")
            device_id = self.login()

        if device_id is not None:
            LOG.debug('Replace URL: \n'
                      'Old URL: %(old_url)s\n,'
                      'New URL: %(new_url)s\n.',
                      {'old_url': old_url,
                       'new_url': self.url})
            result = self.do_call(url, data, method,
                                  log_filter_flag=log_filter_flag)
            if result['error']['code'] in consts.RELOGIN_ERROR_PASS:
                result['error']['code'] = 0
        return result

    def paginated_call(self, url, data=None, method=None,
                       log_filter_flag=False,
                       page_size=consts.QUERY_PAGE_SIZE):
        result_list = []
        start, end = 0, page_size
        msg = _('Query resource volume error')
        while True:
            url_p = '{0}?range=[{1}-{2}]'.format(url, start, end)
            start, end = end, end + page_size
            result = self.call(url_p, data, method, log_filter_flag)
            self._assert_rest_result(result, msg)

            # Empty data if this is first page, OR last page got all data
            if 'data' not in result:
                break

            result_list.extend(result['data'])
            # Check if this is last page
            if len(result['data']) < page_size:
                break

        return result_list

    def logout(self):
        """Logout the session."""
        url = "/sessions"
        if self.url:
            result = self.do_call(url, None, "DELETE")
            self._assert_rest_result(result, _('Logout session error.'))

    def _assert_rest_result(self, result, err_str):
        if result['error']['code'] != 0:
            msg = (_('%(err)s\nresult: %(res)s.') % {'err': err_str,
                                                     'res': result})
            LOG.error(msg)
            raise exception.StorageBackendException(reason=msg)

    def _assert_data_in_result(self, result, msg):
        if 'data' not in result:
            err_msg = _('%s "data" is not in result.') % msg
            LOG.error(err_msg)
            raise exception.StorageBackendException(reason=err_msg)

    def get_storage(self):
        url = "/system/"
        result = self.call(url, method='GET', log_filter_flag=True)

        msg = _('Get storage error.')
        self._assert_rest_result(result, msg)
        self._assert_data_in_result(result, msg)

        return result['data']

    def get_controller(self):
        url = "/controller"
        result = self.call(url, method='GET', log_filter_flag=True)

        msg = _('Get controller error.')
        self._assert_rest_result(result, msg)
        self._assert_data_in_result(result, msg)

        return result['data']

    def get_all_volumes(self):
        url = "/lun"
        return self.paginated_call(url, None, "GET", log_filter_flag=True)

    def get_all_pools(self):
        url = "/storagepool"
        return self.paginated_call(url, None, "GET", log_filter_flag=True)

    def clear_alert(self, sequence_number):
        url = "/alarm/currentalarm?sequence=%s" % sequence_number

        # Result always contains error code and description
        result = self.call(url, method="DELETE", log_filter_flag=True)
        if result['error']['code']:
            msg = 'Clear alert failed with reason: %s.' \
                  % result['error']['description']
            raise exception.InvalidResults(msg)
        return result

    def list_alerts(self):
        url = "/alarm/currentalarm"
        result_list = self.paginated_call(url,
                                          None,
                                          "GET",
                                          log_filter_flag=True)
        return result_list



# Copyright 2020 The SODA Authors.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2010-2011 OpenStack Foundation
# Copyright 2012 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


def fill_storage_attributes(alert_model, storage):
    """ Fills storage attributes for alert model """
    alert_model['storage_id'] = storage['id']
    alert_model['storage_name'] = storage['name']
    alert_model['vendor'] = storage['vendor']
    alert_model['model'] = storage['model']
    alert_model['serial_number'] = storage['serial_number']



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from pyasn1.type.univ import OctetString

from delfin import db, cryptor
from delfin import exception
from delfin.alert_manager import rpcapi
from delfin.api import validation
from delfin.api.common import wsgi
from delfin.api.schemas import alert_source as schema_alert
from delfin.api.views import alert_source as alert_view
from delfin.common import constants

LOG = log.getLogger(__name__)

SNMPv3_keys = ('username', 'auth_key', 'security_level', 'auth_protocol',
               'privacy_protocol', 'privacy_key', 'engine_id')


class AlertSourceController(wsgi.Controller):
    def __init__(self):
        super().__init__()
        self.alert_rpcapi = rpcapi.AlertAPI()

    @wsgi.response(200)
    @validation.schema(schema_alert.put)
    def put(self, req, id, body):
        """Create a new alert source or update an exist one."""
        ctx = req.environ['delfin.context']
        alert_source = body

        alert_source["storage_id"] = id
        db.storage_get(ctx, id)
        alert_source = self._input_check(alert_source)

        snmp_config_to_del = self._get_snmp_config_brief(ctx, id)
        if snmp_config_to_del is not None:
            alert_source = db.alert_source_update(ctx, id, alert_source)
        else:
            alert_source = db.alert_source_create(ctx, alert_source)
        snmp_config_to_add = alert_source
        self.alert_rpcapi.sync_snmp_config(ctx, snmp_config_to_del,
                                           snmp_config_to_add)

        return alert_view.build_alert_source(alert_source.to_dict())

    @wsgi.response(200)
    def show(self, req, id):
        ctx = req.environ['delfin.context']
        alert_source = db.alert_source_get(ctx, id)

        return alert_view.build_alert_source(alert_source.to_dict())

    @wsgi.response(200)
    def delete(self, req, id):
        ctx = req.environ['delfin.context']

        snmp_config_to_del = self._get_snmp_config_brief(ctx, id)
        if snmp_config_to_del is not None:
            self.alert_rpcapi.sync_snmp_config(ctx, snmp_config_to_del,
                                               None)
            db.alert_source_delete(ctx, id)
        else:
            raise exception.AlertSourceNotFound(id)

    def _input_check(self, alert_source):
        version = alert_source.get('version')

        if version.lower() == 'snmpv3':
            user_name = alert_source.get('username')
            security_level = alert_source.get('security_level')
            engine_id = alert_source.get('engine_id')

            # Validate engine_id, check octet string can be formed from it
            if engine_id:
                try:
                    OctetString.fromHexString(engine_id)
                except (TypeError, ValueError):
                    msg = "engine_id should be a set of octets in " \
                          "hexadecimal format."
                    raise exception.InvalidInput(msg)

            if not user_name or not security_level:
                msg = "If snmp version is SNMPv3, then username, " \
                      "security_level are required."
                raise exception.InvalidInput(msg)

            if security_level == constants.SecurityLevel.AUTHNOPRIV\
                    or security_level == constants.SecurityLevel.AUTHPRIV:
                auth_protocol = alert_source.get('auth_protocol')
                auth_key = alert_source.get('auth_key')
                if not auth_protocol or not auth_key:
                    msg = "If snmp version is SNMPv3 and security_level is " \
                          "authPriv or authNoPriv, auth_protocol and " \
                          "auth_key are required."
                    raise exception.InvalidInput(msg)
                alert_source['auth_key'] = cryptor.encode(
                    alert_source['auth_key'])

                if security_level == constants.SecurityLevel.AUTHPRIV:
                    privacy_protocol = alert_source.get('privacy_protocol')
                    privacy_key = alert_source.get('privacy_key')
                    if not privacy_protocol or not privacy_key:
                        msg = "If snmp version is SNMPv3 and security_level" \
                              " is authPriv, privacy_protocol and " \
                              "privacy_key are  required."
                        raise exception.InvalidInput(msg)
                    alert_source['privacy_key'] = cryptor.encode(
                        alert_source['privacy_key'])
                else:
                    alert_source['privacy_key'] = None
                    alert_source['privacy_protocol'] = None
            else:
                alert_source['auth_key'] = None
                alert_source['auth_protocol'] = None
                alert_source['privacy_key'] = None
                alert_source['privacy_protocol'] = None

            # Clear keys for other versions.
            alert_source['community_string'] = None
        else:
            community_string = alert_source.get('community_string')
            if not community_string:
                msg = "If snmp version is SNMPv1 or SNMPv2c, " \
                      "community_string is required."
                raise exception.InvalidInput(msg)
            alert_source['community_string'] = cryptor.encode(
                alert_source['community_string'])

            # Clear keys for SNMPv3
            for k in SNMPv3_keys:
                alert_source[k] = None

        return alert_source

    def _get_snmp_config_brief(self, ctx, storage_id):
        """
        Get snmp configuration that will be used to delete from trap receiver.
        Only community_index(storage_id) required for snmp v1/v2 deletion,
        user_name and engine_id are required for snmp v3. So here we only get
        those required parameters. Return None if configuration not found.
        """
        try:
            alert_source = db.alert_source_get(ctx, storage_id)
            snmp_config = {"storage_id": alert_source["storage_id"],
                           "version": alert_source["version"]}
            if snmp_config["version"].lower() == "snmpv3":
                snmp_config["username"] = alert_source["username"]
                snmp_config["engine_id"] = alert_source["engine_id"]
            return snmp_config
        except exception.AlertSourceNotFound:
            return None

    def _decrypt_auth_key(self, alert_source):
        auth_key = alert_source.get('auth_key', None)
        privacy_key = alert_source.get('privacy_key', None)
        if auth_key:
            alert_source['auth_key'] = cryptor.decode(auth_key)
        if privacy_key:
            alert_source['privacy_key'] = cryptor.decode(privacy_key)

        return alert_source


def create_resource():
    return wsgi.Resource(AlertSourceController())



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# CPG's status
STATUS_POOL_NORMAL = 1  # CPG STATUS Normal operation
STATUS_POOL_DEGRADED = 2  # CPG STATUS Degraded state
STATUS_POOL_FAILED = 3  # CPG STATUS Abnormal operation
STATUS_POOL_UNKNOWN = 99  # CPG STATUS Unknown state
# VOLUME's status
STATUS_VOLUME_NORMAL = 1  # VOLUME STATUS Normal operation
STATUS_VOLUME_DEGRADED = 2  # VOLUME STATUS Degraded state
STATUS_VOLUME_FAILED = 3  # VOLUME STATUS Abnormal operation
STATUS_VOLUME_UNKNOWN = 99  # VOLUME STATUS Unknown state
# VOLUME's type
THIN_LUNTYPE = 2  # TPVV 2	• TPVV,
# VOLUME's Compression status
STATUS_COMPRESSION_YES = 1  # Compression is enabled on the volume
# VOLUME's deduplication status
STATUS_DEDUPLICATIONSTATE_YES = 1  # Enables deduplication on the volume
# MIB to Bytes
MiB_TO_Bytes = 1024 * 1024
# Page size per page at default paging
QUERY_PAGE_SIZE = 150
# Connection timeout
LOGIN_SOCKET_TIMEOUT = 10
SOCKET_TIMEOUT = 10
# 403  The client request has an invalid session key.
# The request came from a different IP address
ERROR_SESSION_INVALID_CODE = 403
# 409  Session key is being used.
ERROR_SESSION_IS_BEING_USED_CODE = 409
# http SUCCESS's status
SUCCESS_STATUS_CODES = 200
# session SUCCESS's status
LOGIN_SUCCESS_STATUS_CODES = 201

# alert state enumeration
ALERT_STATE_NEW = 1  # New.
ALERT_STATE_ACKED = 2  # Acknowledged state.
ALERT_STATE_FIXED = 3  # Alert issue fixed.
ALERT_STATE_UNKNOWN = 99  # Unknown state

# alert severity enumeration
ALERT_SEVERITY_CRITICAL = 2
ALERT_SEVERITY_MAJOR = 3
ALERT_SEVERITY_MINOR = 4
ALERT_SEVERITY_DEGRADED = 5

# alert code
HPE3PAR_ALERT_CODE = {
    '0x0000000': 'Node CPU Thermal Status',
    '0x0010001': 'Serial link event',
    '0x0010002': 'Serial link fail FIFO full',
    '0x0010003': 'Serial link fail full loss',
    '0x0010004': 'Serial link fail rate loss',
    '0x0020001': 'Active VLUN Limit Exceeded',
    '0x0020002': 'System Reporter VLUN performance (major alert)',
    '0x0020003': 'System Reporter VLUN performance (critical alert)',
    '0x0020004': 'System Reporter VLUN performance (minor alert)',
    '0x0020005': 'System Reporter VLUN performance (info alert)',
    '0x0030001': 'Firmware coredump event',
    '0x0030002': 'Too many WWNs on an RCFC port',
    '0x0030003': 'Host [[sw_port]] experienced over 50 CRC '
                 'errors (<count>) in 24 hours',
    '0x0030005': 'FC Port Error',
    '0x0030006': 'FC Port Loop Connection Type Not Supported',
    '0x0030007': 'RCFC port sees non-3PAR WWNs',
    '0x0030009': 'Excessive retransmits on RCFC port',
    '0x0030010': 'Port Device Count Exceeded',
    '0x0030011': 'CRC error on RCIP port',
    '0x0030012': 'Unsupported SATA Drive',
    '0x0030013': 'Unsupported SAS Device',
    '0x0030014': 'Multiple SAS Initiators',
    '0x0030015': 'System Reporter port performance (major alert)',
    '0x0030016': 'Disk Port has exceeded IO error threshold',
    '0x0030017': 'System Reporter port performance (critical alert)',
    '0x0030018': 'System Reporter port performance (minor alert)',
    '0x0030019': 'System Reporter port performance (info alert)',
    '0x00300de': 'Component state change',
    '0x00300fa': 'Component state change',
    '0x0040001': 'Metadata inconsistency in a VV',
    '0x0040003': 'Admin Volume I/O timeout',
    '0x0040004': 'VV availability',
    '0x0040005': 'Pinned DCOWs',
    '0x0040006': 'Aborted DCOWs',
    '0x0040007': 'Recovery scan found corrupt log',
    '0x0040008': 'vlmap count exceeds threshold',
    '0x0040009': 'FlashCache performance degradation',
    '0x004000a': 'VV unrecovered DIF error',
    '0x004000b': 'Metadata inconsistency in a Deduplication Group',
    '0x004000c': 'VV unrecovered DIF error',
    '0x004000d': 'System Reporter VV space major alert',
    '0x004000e': 'System Reporter VV space critical alert',
    '0x004000f': 'System Reporter VV space minor alert',
    '0x0040010': 'System Reporter VV space info alert',
    '0x0040011': 'Flash Cache Creation Failure',
    '0x0040012': 'SD Metadata inconsistency in a VV',
    '0x0040013': 'Compression is not enabled for Volumes less than 16GB',
    '0x0040014': 'System VV detected',
    '0x00400de': 'Component state change',
    '0x00400fa': 'Component state change',
    '0x0050002': 'Ldsk has failed set',
    '0x0050003': 'LD check summary message',
    '0x0050004': 'LD availability has reduced',
    '0x0050005': 'Log LD raid set failure.',
    '0x0050006': 'System Reporter LD performance (major alert)',
    '0x0050007': 'LD check inconsistent',
    '0x0050008': 'LD check failed LD not consistent',
    '0x0050009': 'LD check consistent',
    '0x005000a': 'LD check changed logical disk',
    '0x005000b': 'System Reporter LD performance (critical alert)',
    '0x005000c': 'System Reporter LD performance (minor alert)',
    '0x005000d': 'System Reporter LD performance (info alert)',
    '0x005000f': 'System Reporter LD space critical alert',
    '0x0050010': 'System Reporter LD space minor alert',
    '0x0050011': 'System Reporter LD space info alert',
    '0x0050012': 'System Reporter LD space major alert',
    '0x0060001': 'Disk fail alert',
    '0x0060002': 'Disk monitor stopped',
    '0x0060003': 'Invalid PD configuration',
    '0x0060007': '42 Alerts',
    '0x0060008': 'Disk overtemp warning',
    '0x0060009': 'Disk overtemp alert',
    '0x006000a': 'Chunklet relocation failure',
    '0x006000b': 'System Reporter PD performance (major alert)',
    '0x006000c': 'System overtemp',
    '0x006000d': 'Disk overtemp warning',
    '0x006000e': 'Disk overtemp alert',
    '0x0060011': 'Disk overtemp but not spundown',
    '0x0060012': 'Disk overtemp and spundown',
    '0x0060013': 'Disk overtemp but not spundown no DSK',
    '0x0060014': 'Disk overtemp and spundown no DSK',
    '0x0060015': 'System Reporter PD space major alert',
    '0x0060016': 'System Reporter PD space critical alert',
    '0x0060017': 'System Reporter PD space minor alert',
    '0x0060018': 'System Reporter PD space info alert',
    '0x0060019': 'System Reporter PD performance critical alert',
    '0x006001a': 'System Reporter PD performance minor alert',
    '0x006001b': 'System Reporter PD performance info alert',
    '0x00600de': 'Component state change',
    '0x00600fa': 'Component state change',
    '0x0070001': 'No free chunklet found for relocation',
    '0x0070002': 'No spare chunklet found for relocation',
    '0x0080001': 'Could not process SCSI DB',
    '0x0090001': 'Host Path Status Change',
    '0x00900de': 'Component state change',
    '0x00a0005': 'Snap Admin Volume low on space, degraded',
    '0x00a0006': 'Snap Data Volume low on space, degraded',
    '0x00a0007': 'Second snap Data Volume low on space, degraded',
    '0x00b0001': 'Kernel crashdump event',
    '0x00b0002': 'Kernel crashdump with error',
    '0x00c0001': 'Process has exited',
    '0x00c0002': 'Process cannot be started',
    '0x00c0003': 'Process coredump event',
    '0x00c0004': 'Attempt to run grub failed',
    '0x00c0005': 'Attempt to run grub failed, PM not starting',
    '0x00c0006': 'Attempt to run grub failed, retval',
    '0x00c0007': 'Process coredump with error',
    '0x00d0001': 'Corrupt PR table found',
    '0x00d0002': 'PR transition',
    '0x00d0003': 'PR transition, degraded.',
    '0x00e0001': 'Double node failure',
    '0x00e0002': 'System manager cannot startup',
    '0x00e0003': 'Node recovery powerfail event',
    '0x00e0004': '<success> use of golden license',
    '0x00e0005': 'License key usage, license expired',
    '0x00e0006': 'System recovery notification about bad volume',
    '0x00e0007': 'Pfail partition needs to be wiped',
    '0x00e0008': 'Power fail saved version mismatch',
    '0x00e0009': 'Failed to save task data',
    '0x00e000a': 'Task failed',
    '0x00e000b': 'Pfail recovery continued with failed previous NM1 recovery',
    '0x00e000d': 'System recovery stalled due to unknown replicant state',
    '0x00e000e': 'System recovery stalled due to sole owner of ld missing',
    '0x00e0011': '"servicemag start" operation has completed',
    '0x00e0012': '"servicemag resume" operation has completed',
    '0x00e0014': 'Battery States',
    '0x00e0015': 'Node not integrated',
    '0x00e0016': 'System recovery stalled due to unstarted vvs',
    '0x00e0017': 'TOC corruption detected',
    '0x00e0018': 'Pfail Recovery with a missing VV',
    '0x00e0019': 'Pfail Recovery with VV in bad state',
    '0x00e001a': 'Pfail Recovery skipped due to multiple NM1 nodes',
    '0x00e001b': 'NM1 pfail recovery proceeding with missing replicant',
    '0x00e001c': 'Configuration lock hold time',
    '0x00e001d': 'Inconsistent TOC object removed',
    '0x00e001e': 'Invalid VVMEMB(s) resolved',
    '0x00e001f': '"servicemag resume" operation has passed '
                 'with dismissed disks',
    '0x00e0020': '"servicemag resume" operation has passed '
                 'without dismissing any disks',
    '0x00e0021': '"servicemag resume" operation has failed '
                 'with no error message',
    '0x00e0022': '"servicemag resume" operation has failed to admit disk',
    '0x00e0023': '"servicemag resume" operation has failed '
                 'unrecoverable disk',
    '0x00e0024': '"servicemag resume" operation has failed to '
                 'relocate_chunklets',
    '0x00e0025': 'System manager cannot start up, TOC not found',
    '0x00e0026': 'System manager cannot start up, waiting on nodes',
    '0x00e0027': 'System manager cannot start up, manual start up set',
    '0x00e0028': 'System manager cannot start up, TOC quorum not met',
    '0x00e0029': 'System manager cannot start up, waiting for '
                 'nodes to recover',
    '0x00e002a': 'Pfail partition needs to be wiped',
    '0x00e002b': 'Pfail partition needs to be wiped',
    '0x00e002c': 'System manager cannot start up, incomplete powerfail',
    '0x00e002d': 'System manager cannot start up, TOC quorum found, '
                 'incomplete powerfail',
    '0x00e002e': 'System manager cannot start up, TOC quorum found, '
                 'waiting for nodes to recover',
    '0x00e002f': 'System manager cannot start up, waiting for nodes '
                 'to recover',
    '0x00e0030': 'Unexpected encryption state on node drive',
    '0x00e0031': '"servicemag start" failed',
    '0x00e0032': 'Single node WBC is active',
    '0x00e0033': 'Single node WBC is expired',
    '0x0100001': 'Online upgrade',
    '0x0100002': 'Unresponsive IOCTL',
    '0x0100003': 'Update available',
    '0x0100004': 'Update status',
    '0x0100005': 'Update install status',
    '0x0100006': 'Unresponsive IOCTL Verbose',
    '0x0110001': 'Errors accessing the IDE disk',
    '0x0110002': 'IDE disk error handling',
    '0x0110004': 'Version mismatch event',
    '0x0110005': 'Serial comm init failed',
    '0x0110006': 'IDE disk error node shutdown',
    '0x0110007': 'IDE disk error node not shutdown',
    '0x0110008': 'IDE disk error node not shutdown LDs cannot be served',
    '0x0110009': 'IDE disk error node reboot',
    '0x011000a': 'Version mismatch event for svcalert',
    '0x011000b': 'Version mismatch event',
    '0x011000c': 'Version mismatch event',
    '0x0130001': 'Too many alerts in the system',
    '0x0140001': 'Notification',
    '0x0140003': 'fork(2) call failed',
    '0x0140004': 'System Reporter QoS performance (major alert)',
    '0x0140005': 'SFP Unqualified Notification',
    '0x0140007': 'System upgrade cancelled',
    '0x0140008': 'System upgrade Cancellation Failed',
    '0x0140009': 'System serial number could not be determined',
    '0x014000a': 'DC3 I2C Lockup Reset Succeeded',
    '0x014000b': 'DC3 I2C Lockup Reset Failed',
    '0x014000c': 'admitpd not allowed on Emulex generated wwn',
    '0x014000d': 'admitpd not allowed on toto-sata generated wwn',
    '0x014000e': 'RAID 0 LD failed due to stale chunklet',
    '0x014000f': 'Mismatch of failed chunklet information',
    '0x0140010': 'System Reporter QoS performance (critical alert)',
    '0x0140011': 'System Reporter QoS performance (minor alert)',
    '0x0140012': 'System Reporter QoS performance (info alert)',
    '0x0150004': 'CLI server cannot communicate with system manager',
    '0x0150005': 'CLI internal error using authentication library',
    '0x0150006': 'Authentication failure',
    '0x0150007': 'CLI internal error',
    '0x015000c': 'CPG free space limit',
    '0x015000d': 'CLI client process event',
    '0x015000f': 'Relocatepd request',
    '0x0150010': 'Control Recovery Auth Ciphertext Export',
    '0x0150011': 'CLI server process event, max tpdtcl exceeded',
    '0x0150012': 'CLI server process event, twice max tpdtcl exceeded',
    '0x0150013': 'CLI server process event, max CLI server exceeded',
    '0x0150014': 'CLI server process event, max local exceeded',
    '0x0150015': 'CLI server process event, max server exceeded brief',
    '0x0150016': 'CLI server process event, max server exceeded local',
    '0x0150017': 'CLI server process event, error in track',
    '0x0150018': 'CLI server process event, error in store user name',
    '0x0150019': 'CLI server process event, svcalert brief',
    '0x015001a': 'CLI server process event, svcalert',
    '0x015001b': 'CLI internal error Failed sanity check',
    '0x015001c': 'CLI internal error sqlite database',
    '0x015001d': 'CLI internal error SQLite DB',
    '0x015001f': 'CLI client process event disk high temp',
    '0x0150020': 'Unable to send an event to the security syslog server.',
    '0x0150021': 'Connection has been reestablished to the '
                 'security syslog server.',
    '0x0150022': 'Slow Disk temperature unavailable',
    '0x0170001': 'TOC update',
    '0x0170004': 'TOC update, not above error threshold and decreased.',
    '0x0170005': 'TOC update, not above warn threshold and decreased.',
    '0x0190001': 'ea msg timeout',
    '0x0190002': 'Pre Integration Link Test Error',
    '0x01a0001': 'CPU Memory Correctable ECC',
    '0x01a0002': 'Node is offline',
    '0x01a0003': 'Node Time of Day Battery',
    '0x01a0005': 'HW: CPU Memory Correctable ECC',
    '0x01a0006': 'CPU Configuration',
    '0x01a0007': 'BIOS IDE log entry',
    '0x01a0008': 'Node Environmental Check Pass',
    '0x01a0009': 'IDE file integrity check results',
    '0x01a000b': 'Eagle memory uerr',
    '0x01a000c': 'Eagle memory muerr',
    '0x01a000d': 'Eagle memory cerr',
    '0x01a000e': 'Eagle internal system error',
    '0x01a000f': 'Eagle hardware watchdog error',
    '0x01a0010': 'Eagle PCI error',
    '0x01a0011': 'Eagle driver software error',
    '0x01a0012': 'Memory usage information',
    '0x01a0014': 'Too many TCP segment retransmits',
    '0x01a0015': 'Node PCIe Correctable Error Status',
    '0x01a0016': 'Node PCIe Link Status',
    '0x01a0017': 'Too many TCP segment errors',
    '0x01a0019': 'Cluster thermal shutdown',
    '0x01a001a': 'Link Configuration Mismatch',
    '0x01a001b': 'Unexpected Cable Event',
    '0x01a001c': 'Link establish alert',
    '0x01a001d': 'Core File Received From Remote/Local MCU',
    '0x01a001f': 'Node Needs to Shutdown',
    '0x01a0021': 'Node Rescue',
    '0x01a0022': 'Node-Failure-Analysis File Received From Remote/Local MCU',
    '0x01a0024': 'Slab usage information',
    '0x01a0025': 'System Reporter cmp performance (major alert)',
    '0x01a0026': 'System Reporter CPU performance (major alert)',
    '0x01a0027': 'System Reporter link performance (major alert)',
    '0x01a0028': 'Node ID Mismatch',
    '0x01a0029': 'Remote Node ID Mismatch',
    '0x01a002a': 'System Model Mismatch',
    '0x01a002b': 'Remote System Model Mismatch',
    '0x01a002c': 'Node Type Mismatch',
    '0x01a002d': 'Remote Node Type Mismatch',
    '0x01a002e': 'SSN Mismatch',
    '0x01a002f': 'Remote SSN Mismatch',
    '0x01a0031': 'Node Rescue User Abort',
    '0x01a0032': 'Node Rescue Invalid',
    '0x01a0033': 'Node Rescue Internal Communication Error',
    '0x01a0034': 'Node Rescue No Rejoin',
    '0x01a0035': 'Node Rescue Port 80 Blocked',
    '0x01a0036': 'Node Rescue Port 69 Blocked',
    '0x01a0037': 'Node Rescue Port 873 Blocked',
    '0x01a0038': 'Node Rescue No Backplane Connection',
    '0x01a0039': 'CMP Threshold',
    '0x01a003a': 'DIF error',
    '0x01a003b': 'IDE file integrity check bad run',
    '0x01a003c': 'IDE file integrity check bad',
    '0x01a003d': 'IDE file integrity check very bad',
    '0x01a003e': 'System Reporter cache performance alert',
    '0x01a003f': 'Legacy System Model Mismatch',
    '0x01a0040': 'Remote System Model Mismatch',
    '0x01a0041': 'Node Rescue Detected Dual Boot Node Drive Size Mismatch',
    '0x01a0042': 'Node Environmental Check Fail',
    '0x01a0043': 'Node Thermal Status svc alert',
    '0x01a0044': 'Node Needs to Shutdown svc alert',
    '0x01a0045': 'Node Thermal Status Alert',
    '0x01a0046': 'Node Thermal Status Warning',
    '0x01a0047': 'System Reporter cmp performance (critical alert)',
    '0x01a0048': 'System Reporter cmp performance (minor alert)',
    '0x01a0049': 'System Reporter cmp performance (info alert)',
    '0x01a004a': 'System Reporter CPU performance (critical alert)',
    '0x01a004b': 'System Reporter CPU performance (minor alert)',
    '0x01a004c': 'System Reporter CPU performance (info alert)',
    '0x01a004d': 'System Reporter link performance (critical alert)',
    '0x01a004e': 'System Reporter link performance (minor alert)',
    '0x01a004f': 'System Reporter link performance (info alert)',
    '0x01a0050': 'System Reporter cache performance (critical alert)',
    '0x01a0051': 'System Reporter cache performance (minor alert)',
    '0x01a0052': 'System Reporter cache performance (info alert)',
    '0x01a0053': 'Eagle link error',
    '0x01a0054': 'System Series Mismatch',
    '0x01a0055': 'Remote System Series Mismatch',
    '0x01a0056': 'Node temporary filesystem in use',
    '0x01a0057': 'Node rescue detected that rescuee node has an '
                 'incompatible board series',
    '0x01a00de': 'Component state change',
    '0x01a00fa': 'Component state change',
    '0x01b0001': 'Power Supply',
    '0x01b0002': 'Power Supply DC Status',
    '0x01b0003': 'Power Supply AC Status',
    '0x01b0004': 'Power Supply Fan Status',
    '0x01b0005': 'Power Supply Charger Status',
    '0x01b0009': 'Power Supply Type Mismatch',
    '0x01b0015': 'VSC 055 Interrupt Error',
    '0x01b00de': 'Component state change',
    '0x01b00fa': 'Component state change',
    '0x01d0001': 'Bios eeprom log events',
    '0x01e0001': 'Cage log event',
    '0x01e0005': 'Cage coredump event',
    '0x01e0006': 'servicemag failed to dismiss PD: '
                 'cage <cageid>, mag <magid>, '
                 'taskid <taskid>, pd <pdid>: error<smag_err> - <text>',
    '0x01e0007': 'Critical ESI port count, down to one',
    '0x01e0008': 'Critical ESI port count, one valid',
    '0x01e0009': 'Critical ESI port count, lost',
    '0x01e000a': 'Invalid cage isolated configuration',
    '0x01e000b': 'Invalid cage isolated configuration',
    '0x01e000c': 'Invalid cage mixed configuration',
    '0x01e000d': 'Invalid cage unknown configuration',
    '0x01e000e': 'Invalid cage partners configuration',
    '0x01e000f': 'Invalid cage maxcage configuration',
    '0x01e0010': 'Invalid cage twice configuration',
    '0x01e0011': 'Unknown cage configuration',
    '0x01e0012': 'Cage coredump event - detailed - 0',
    '0x01e0013': 'Cage coredump event - detailed - 1',
    '0x01e0014': 'Cage coredump event - detailed - 2',
    '0x01e0015': 'Cage coredump event - detailed - 3',
    '0x01e0016': 'Cage coredump event - very detailed - 0',
    '0x01e0017': 'Cage coredump event - very detailed - 1',
    '0x01e0018': 'Cage log event, firmware panic',
    '0x01e0019': 'Cage log event, midplane esi',
    '0x01e001a': 'Cage log event, midplane',
    '0x01e001b': 'Cage log event, post',
    '0x01e001c': 'Cage log event, midplane lm87',
    '0x01e001d': 'Cage log event, midplane pmc',
    '0x01e00de': 'Component state change',
    '0x01e00fa': 'Component state change',
    '0x01f0001': 'Mixing SSDs with different RPMs not supported',
    '0x01f00de': 'Component state change',
    '0x01f00fa': 'Component state change',
    '0x0200006': 'GUI server can not communicate with the system manager',
    '0x0200009': 'Internal error in authentication library',
    '0x0210001': 'InForm GUI has lost connection to the event filter',
    '0x0220001': 'Battery expiring soon',
    '0x0220010': 'Assert Battery FAIL',
    '0x0220014': 'Battery Type Mismatch',
    '0x0220017': 'Battery expiration soon',
    '0x02200de': 'Component state change',
    '0x02200fa': 'Component state change',
    '0x0230003': 'Port shutdown on fatal error',
    '0x0230004': 'Host port is down',
    '0x0230005': 'All ports in the same FC card must be configured for RCFC',
    '0x0230006': 'HBA fw file status',
    '0x0230007': 'HBA FW error opening file',
    '0x0230008': 'HBA FW error reading file',
    '0x0230009': 'HBA FW unsupported file',
    '0x0240002': 'Internodal Serial Port Receiver Timeout Error',
    '0x0240003': 'Internodal Serial Port Default Error',
    '0x0250002': 'Remote Copy link status',
    '0x0250007': 'System Reporter RC Target performance (major alert)',
    '0x0250008': 'System Reporter RC VV performance (major alert)',
    '0x0250009': 'Remote Copy group in failsafe state',
    '0x025000a': 'Replication resource usage exceeded - Group "Logging".',
    '0x025000b': 'Replication resource usage exceeded - Group "Stopped".',
    '0x025000c': 'Replication resources restored - Group transition '
                 'from Logging failure',
    '0x025000d': 'System Reporter RC VV performance (critical alert)',
    '0x025000e': 'System Reporter RC VV performance (minor alert)',
    '0x025000f': 'System Reporter RC VV performance (info alert)',
    '0x0250011': 'System Reporter RC Target performance (critical alert)',
    '0x0250012': 'System Reporter RC Target performance (minor alert)',
    '0x0250013': 'System Reporter RC Target performance (info alert)',
    '0x0250014': 'Remote Copy group status alert',
    '0x0250015': 'Remote Copy group status fail',
    '0x0250016': 'Quorum is not in Started state',
    '0x0260001': 'Ethernet Monitor Event',
    '0x0260002': 'No admin network interface discovered',
    '0x0270001': 'TP VV allocation size warning',
    '0x0270002': 'TP VV allocation size limit',
    '0x0270003': 'Snapshot space allocation size warning',
    '0x0270004': 'Snapshot space allocation size limit',
    '0x0270005': 'CPG growth warning',
    '0x0270006': 'CPG growth limit',
    '0x0270007': 'TP VV allocation failure',
    '0x0270008': 'Snapshot space allocation failure',
    '0x0270009': 'CPG growth failure',
    '0x027000e': 'FC raw space allocation 50% alert',
    '0x027000f': 'FC raw space allocation 75% alert',
    '0x0270010': 'FC raw space allocation 85% alert',
    '0x0270011': 'FC raw space allocation 95% alert',
    '0x0270012': 'CPG space used status',
    '0x0270013': 'Raw space allocation user configured alert',
    '0x0270014': 'NL raw space allocation 50% alert',
    '0x0270015': 'NL raw space allocation 75% alert',
    '0x0270016': 'NL raw space allocation 85% alert',
    '0x0270017': 'NL raw space allocation 95% alert',
    '0x0270018': 'CPG was grown with degraded parameters',
    '0x0270019': 'SSD raw space allocation 50% alert',
    '0x027001a': 'SSD raw space allocation 75% alert',
    '0x027001b': 'SSD raw space allocation 85% alert',
    '0x027001c': 'SSD raw space allocation 95% alert',
    '0x027001d': 'CPG growth failure non-admin',
    '0x027001e': 'CPG growth non admin limit',
    '0x027001f': 'CPG growth non admin warning',
    '0x0270020': 'Overprovisioning CPG warning alert',
    '0x0270021': 'Overprovisioning CPG limit alert',
    '0x0270022': 'Overprovisioning warning alert',
    '0x0270023': 'Overprovisioning limit alert',
    '0x0270024': 'System Reporter CPG space critical alert',
    '0x0270025': 'System Reporter CPG space minor alert',
    '0x0270026': 'System Reporter CPG space info alert',
    '0x0270027': 'System Reporter CPG space major alert',
    '0x0280001': 'Preserved data LDs configuration',
    '0x0280002': 'Preserved data LDs unavailable',
    '0x0280003': 'Preserved data LDs are filling up',
    '0x0280004': 'Preserved data LDs are full',
    '0x0280005': 'LD availability',
    '0x0280006': 'Preserved data LDs status, mangler class',
    '0x0280007': 'Preserved data LDs configuration, Not configured',
    '0x0280008': 'Preserved data LDs configuration, Not started',
    '0x02900de': 'Component state change',
    '0x02a00de': 'Component state change',
    '0x02a00fa': 'Component state change',
    '0x02b00de': 'Component state change',
    '0x02b00fa': 'Component state change',
    '0x02d00de': 'Component state change',
    '0x02d00fa': 'Component state change',
    '0x03500de': 'Component state change',
    '0x03500fa': 'Component state change',
    '0x0360002': 'Write Cache Availability',
    '0x0360003': 'System Reporter system space critical alert',
    '0x0360004': 'System Reporter system space major alert',
    '0x0360005': 'System Reporter system space info alert',
    '0x0360006': 'System Reporter system space minor alert',
    '0x03700de': 'Component state change',
    '0x03700fa': 'Component state change',
    '0x03800de': 'Component state change',
    '0x03900fa': 'Component state change',
    '0x03a00de': 'Component state change',
    '0x03a00fa': 'Component state change',
    '0x03b0002': 'Free node disk space low',
    '0x03b0004': 'Node drive is encrypted but encryption is '
                 'not enabled on the system',
    '0x03b0005': 'Encryption is enabled on the system but the '
                 'node drive is not encrypted',
    '0x03b0006': 'Unable to do I/O to the node drive',
    '0x03b0007': 'Free node disk space low, /common not mounted',
    '0x03b0008': 'Free node disk space low, /altroot not mounted',
    '0x03b0009': 'Free node disk space low, /common and /altroot not mounted',
    '0x03b000a': 'Syslog Node Drive Failure Message Monitoring',
    '0x03b000b': 'Periodic /proc/mdstat Monitoring '
                 'Detected Degraded Node Drive Raid',
    '0x03b000c': 'Lost interrupt',
    '0x03b000d': 'IDE SMART failed self check',
    '0x03b000e': 'IDE SMART unreadable sectors',
    '0x03b000f': 'IDE SMART uncorrectable sectors',
    '0x03b0010': 'IDE SMART failed unit ready',
    '0x03b0011': 'IDE SMART failed usage attribute',
    '0x03b0012': 'IDE SMART failure',
    '0x03b0013': 'IDE SMART execute test failed',
    '0x03b0014': 'IDE SMART new self test log error',
    '0x03b0015': 'IDE SMART repeat self test log error',
    '0x03b0016': 'IDE SMART ATA error increase',
    '0x03b0017': 'IDE SMART attribute data read fail',
    '0x03b0019': 'IDE SMART error log read fail',
    '0x03b0020': 'DUAL IDE SMART failed self check',
    '0x03b0021': 'DUAL IDE SMART unreadable sectors',
    '0x03b0022': 'DUAL IDE SMART uncorrectable sectors',
    '0x03b0023': 'DUAL IDE SMART failed unit ready',
    '0x03b0024': 'DUAL IDE SMART failed usage attribute',
    '0x03b0025': 'DUAL IDE SMART failure',
    '0x03b0026': 'DUAL IDE SMART execute test failed',
    '0x03b0027': 'DUAL IDE SMART new self test log error',
    '0x03b0028': 'DUAL IDE SMART repeat self test log error',
    '0x03b0029': 'DUAL IDE SMART ATA error increase',
    '0x03b002a': 'DUAL IDE SMART attribute data read fail',
    '0x03b002b': 'DUAL IDE SMART error log read fail',
    '0x03f0001': 'Process appears unresponsive',
    '0x03f0002': 'Process name appears unresponsive',
    '0x03f0003': 'Process event handling appears unresponsive',
    '0x0450001': 'Data Cache DIMM CECC Monitoring',
    '0x0450002': 'Patrol Data Cache DIMM UERR',
    '0x0460001': 'Control Cache DIMM Temperature',
    '0x0460002': 'Control Cache DIMM Temperature',
    '0x0460003': 'Node FB-DIMM AMB Correctable Error Status',
    '0x04a0001': 'Slot PCIe Correctable Error Status',
    '0x04a0002': 'Slot PCIe Link Status',
    '0x04e0001': 'Rejecting SSH Connection',
    '0x04e0002': 'Rejecting SSH Connection from IP',
    '0x0500001': 'A system task failed',
    '0x05d00de': 'Component state change',
    '0x05d00fa': 'Component state change',
    '0x0600005': 'WSAPI internal error using authentication library',
    '0x06200fa': 'Component state change',
    '0x0640001': 'PD Scrub',
    '0x0660001': 'SED is from the wrong system',
    '0x0660002': 'SED has the wrong key',
    '0x0660003': 'SED is present, but encryption is not enabled',
    '0x0660004': 'LKM is in an unknown state',
    '0x0660005': 'MMAP failed to map the segment of the memory with keys',
    '0x0660006': 'Nodesvr unresponsive during darsvr startup',
    '0x0660007': 'Nodesvr unresponsive during fipsvr startup',
    '0x0660008': 'fipsvr unable to start in FIPS mode',
    '0x0660009': 'Failed to successfully communicate with EKM at startup',
    '0x066000a': 'Controlencryption restore failed',
    '0x066000b': 'Controlencryption restore ignore failed',
    '0x066000c': 'Controlencryption restore ignore succeeded with failures',
    '0x066000d': 'Encryption operation attempted on drive with WWN 0',
    '0x066000e': 'Unsupported drive present in the system',
    '0x06700de': 'Component state change',
    '0x0680001': 'Quorum Witness',
    '0x06e0001': 'File Services state change',
    '0x0720001': 'File Provisioning Group',
    '0x0740001': 'File Store',
    '0x0750001': 'Virtual Server IP Address',
    '0x0760001': 'Node Network Bond',
    '0x0770001': 'Node Network Interface',
    '0x0780001': 'Node IP Address',
    '0x0790001': 'File Service Node Active Directory Configuration',
    '0x07e0001': 'Anti-Virus VSE Server',
    '0x0810001': 'Anti-Virus Scan',
    '0x0820001': 'Virtual Server Certificate',
    '0x0840001': 'HTTP Share',
    '0x0850001': 'NFS Share',
    '0x0860001': 'SMB Share',
    '0x0870001': 'User Quota',
    '0x08b0001': 'File Store Snapshot',
    '0x08c0001': 'File Provisioning Group Snap Reclamation Task',
    '0x08d0001': 'Overall File Services for Node',
    '0x08e0001': 'File Services Software Update',
    '0x08f0001': 'File Services Log Collection',
    '0x0900001': 'File Service Virtual Server Backup',
    '0x0960002': 'Vasa Provider migration failed due to VVol SC migration',
    '0x0960003': 'Vasa Provider migration failed due '
                 'to Certificate mode migration',
    '0x0960004': 'Vasa Provider migration failed while updating config file',
    '0x0960005': 'VASA provider could not start because of '
                 'issues with the VASA Certificate',
    '0x0990001': 'Static IP Route',
    '0x09a0001': 'SMB Global Setting State change event',
    '0x09b0001': 'Ddcscan Monitoring',
    '0x09d0001': 'NVDIMM Battery Failure',
    '0x09e0003': 'Management Module High Temperature',
    '0x09e0004': 'Management Module not responding',
    '0x09f0001': 'File Persona VM shutdown',
    '0x09f0002': 'File Persona CPG grow limit warning',
    '0x0a50001': 'File Access Auditing Alerts'
}



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import binascii
from datetime import datetime

import six
from oslo_config import cfg
from oslo_log import log
from pyasn1.type.univ import OctetString
from pysnmp.entity.rfc3413.oneliner import cmdgen

from delfin import cryptor
from delfin import db
from delfin import exception
from delfin.common import constants
from delfin.exporter import base_exporter

CONF = cfg.CONF

LOG = log.getLogger(__name__)


class SNMPValidator(object):
    def __init__(self):
        self.exporter = base_exporter.AlertExporterManager()
        self.snmp_error_flag = {}

    def validate(self, ctxt, alert_source):
        alert_source = dict(alert_source)
        engine_id = alert_source.get('engine_id')
        try:
            alert_source = self.validate_connectivity(alert_source)

            # If protocol is snmpv3, the snmp_validator will update
            # engine id if engine id is empty. Therefore, engine id
            # should be saved in database.
            if not engine_id and alert_source.get('engine_id'):

                alert_source_dict = {
                    'engine_id': alert_source.get('engine_id')}
                db.alert_source_update(ctxt,
                                       alert_source.get('storage_id'),
                                       alert_source_dict)
            self._handle_validation_result(ctxt,
                                           alert_source.get('storage_id'),
                                           constants.Category.RECOVERY)
        except exception.SNMPConnectionFailed:
            self._handle_validation_result(ctxt,
                                           alert_source.get('storage_id'))
        except Exception as e:
            msg = six.text_type(e)
            LOG.error("Failed to check snmp config. Reason: %s", msg)

    @staticmethod
    def validate_connectivity(alert_source):
        # Fill optional parameters with default values if not set in input
        if not alert_source.get('port'):
            alert_source['port'] = constants.DEFAULT_SNMP_CONNECT_PORT

        if not alert_source.get('context_name'):
            alert_source['context_name'] = None

        if not alert_source.get('retry_num'):
            alert_source['retry_num'] = constants.DEFAULT_SNMP_RETRY_NUM

        if not alert_source.get('expiration'):
            alert_source['expiration'] = constants.DEFAULT_SNMP_EXPIRATION_TIME

        if CONF.snmp_validation_enabled is False:
            return alert_source

        cmd_gen = cmdgen.CommandGenerator()

        # Register engine observer to get engineId,
        # Code reference from: http://snmplabs.com/pysnmp/
        observer_context = {}
        cmd_gen.snmpEngine.observer.registerObserver(
            lambda e, p, v, c: c.update(
                securityEngineId=v['securityEngineId']),
            'rfc3412.prepareDataElements:internal',
            cbCtx=observer_context
        )

        version = alert_source.get('version')

        # Connect to alert source through snmp get to check the configuration
        try:
            if version.lower() == 'snmpv3':
                auth_key = cryptor.decode(alert_source['auth_key'])
                privacy_key = cryptor.decode(alert_source['privacy_key'])
                auth_protocol = None
                privacy_protocol = None
                if alert_source['auth_protocol']:
                    auth_protocol = constants.AUTH_PROTOCOL_MAP.get(
                        alert_source['auth_protocol'].lower())
                if alert_source['privacy_protocol']:
                    privacy_protocol = constants.PRIVACY_PROTOCOL_MAP.get(
                        alert_source['privacy_protocol'].lower())

                engine_id = alert_source.get('engine_id')
                if engine_id:
                    engine_id = OctetString.fromHexString(engine_id)
                error_indication, __, __, __ = cmd_gen.getCmd(
                    cmdgen.UsmUserData(alert_source['username'],
                                       authKey=auth_key,
                                       privKey=privacy_key,
                                       authProtocol=auth_protocol,
                                       privProtocol=privacy_protocol,
                                       securityEngineId=engine_id),
                    cmdgen.UdpTransportTarget((alert_source['host'],
                                               alert_source['port']),
                                              timeout=alert_source[
                                                  'expiration'],
                                              retries=alert_source[
                                                  'retry_num']),
                    constants.SNMP_QUERY_OID,
                )

                if 'securityEngineId' in observer_context:
                    engine_id = observer_context.get('securityEngineId')
                    alert_source['engine_id'] = binascii.hexlify(
                        engine_id.asOctets()).decode()
            else:
                community_string = cryptor.decode(
                    alert_source['community_string'])
                error_indication, __, __, __ = cmd_gen.getCmd(
                    cmdgen.CommunityData(
                        community_string,
                        contextName=alert_source['context_name']),
                    cmdgen.UdpTransportTarget((alert_source['host'],
                                               alert_source['port']),
                                              timeout=alert_source[
                                                  'expiration'],
                                              retries=alert_source[
                                                  'retry_num']),
                    constants.SNMP_QUERY_OID,
                )

            if not error_indication:
                return alert_source

            # Prepare exception with error_indication
            msg = six.text_type(error_indication)
        except Exception as e:
            msg = six.text_type(e)

        # Since validation occur error, raise exception
        LOG.error("Configuration validation failed with alert source for "
                  "reason: %s." % msg)
        raise exception.SNMPConnectionFailed(msg)

    def _handle_validation_result(self, ctxt, storage_id,
                                  category=constants.Category.FAULT):
        try:
            storage = db.storage_get(ctxt, storage_id)
            serial_number = storage.get('serial_number')
            if category == constants.Category.FAULT:
                self.snmp_error_flag[serial_number] = True
                self._dispatch_snmp_validation_alert(ctxt, storage, category)
            elif self.snmp_error_flag.get(serial_number, True):
                self.snmp_error_flag[serial_number] = False
                self._dispatch_snmp_validation_alert(ctxt, storage, category)
        except Exception as e:
            msg = six.text_type(e)
            LOG.error("Exception occurred when handling validation "
                      "error: %s ." % msg)

    def _dispatch_snmp_validation_alert(self, ctxt, storage, category):

        alert = {
            'storage_id': storage['id'],
            'storage_name': storage['name'],
            'vendor': storage['vendor'],
            'model': storage['model'],
            'serial_number': storage['serial_number'],
            'alert_id': constants.SNMP_CONNECTION_FAILED_ALERT_ID,
            'sequence_number': 0,
            'alert_name': 'SNMP connect failed',
            'category': category,
            'severity': constants.Severity.MAJOR,
            'type': constants.EventType.COMMUNICATIONS_ALARM,
            'location': 'NetworkEntity=%s' % storage['name'],
            'description': "SNMP connection to the storage failed. "
                           "SNMP traps from storage will not be received.",
            'recovery_advice': "1. The network connection is abnormal. "
                               "2. SNMP authentication parameters "
                               "are invalid.",
            'occur_time': int(datetime.utcnow().timestamp()) * 1000,
        }
        self.exporter.dispatch(ctxt, alert)



# Copyright 2020 The SODA Authors.
# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Defines interface for DB access.

The underlying driver is loaded as a :class:`LazyPluggable`.

Functions in this module are imported into the delfin.db namespace. Call these
functions from delfin.db namespace, not the delfin.db.api namespace.

All functions in this module return objects that implement a dictionary-like
interface. Currently, many of these objects are sqlalchemy objects that
implement a dictionary interface. However, a future goal is to have all of
these objects be simple dictionaries.


**Related Flags**

:backend:  string to lookup in the list of LazyPluggable backends.
           `sqlalchemy` is the only supported backend right now.

:connection:  string specifying the sqlalchemy connection to use, like:
              `sqlite:///var/lib/delfin/delfin.sqlite`.

:enable_new_services:  when adding a new service to the database, is it in the
                       storage_pool of available hardware (Default: True)

"""
from oslo_config import cfg
from oslo_db import api as db_api

db_opts = [
    cfg.StrOpt('db_backend',
               default='sqlalchemy',
               help='The backend to use for database.'),
]

CONF = cfg.CONF
CONF.register_opts(db_opts, "database")

_BACKEND_MAPPING = {'sqlalchemy': 'delfin.db.sqlalchemy.api'}
IMPL = db_api.DBAPI(CONF.database.db_backend, backend_mapping=_BACKEND_MAPPING,
                    lazy=True)


def register_db():
    """Create database and tables."""
    IMPL.register_db()


def storage_get(context, storage_id):
    """Retrieve a storage device."""
    return IMPL.storage_get(context, storage_id)


def storage_get_all(context, marker=None, limit=None, sort_keys=None,
                    sort_dirs=None, filters=None, offset=None):
    """Retrieves all storage devices.

    If no sort parameters are specified then the returned volumes are sorted
    first by the 'created_at' key and then by the 'id' key in descending
    order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of storage
    """
    return IMPL.storage_get_all(context, marker, limit, sort_keys, sort_dirs,
                                filters, offset)


def storage_create(context, values):
    """Add a storage device from the values dictionary."""
    return IMPL.storage_create(context, values)


def storage_update(context, storage_id, values):
    """Update a storage device with the values dictionary."""
    return IMPL.storage_update(context, storage_id, values)


def storage_delete(context, storage_id):
    """Delete a storage device."""
    return IMPL.storage_delete(context, storage_id)


def volume_create(context, values):
    """Create a volume from the values dictionary."""
    return IMPL.volume_create(context, values)


def volumes_create(context, values):
    """Create multiple volumes."""
    return IMPL.volumes_create(context, values)


def volume_update(context, volume_id, values):
    """Update a volume with the values dictionary."""
    return IMPL.volume_update(context, volume_id, values)


def volumes_update(context, values):
    """Update multiple volumes."""
    return IMPL.volumes_update(context, values)


def volumes_delete(context, values):
    """Delete multiple volumes."""
    return IMPL.volumes_delete(context, values)


def volume_get(context, volume_id):
    """Get a volume or raise an exception if it does not exist."""
    return IMPL.volume_get(context, volume_id)


def volume_get_all(context, marker=None, limit=None, sort_keys=None,
                   sort_dirs=None, filters=None, offset=None):
    """Retrieves all volumes.

    If no sort parameters are specified then the returned volumes are sorted
    first by the 'created_at' key and then by the 'id' key in descending
    order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of volumes
    """
    return IMPL.volume_get_all(context, marker, limit, sort_keys,
                               sort_dirs, filters, offset)


def volume_delete_by_storage(context, storage_id):
    """Delete all the volumes of a device."""
    return IMPL.volume_delete_by_storage(context, storage_id)


def storage_pool_create(context, storage_pool):
    """Add a storage_storage_pool."""
    return IMPL.storage_pool_create(context, storage_pool)


def storage_pools_create(context, storage_pools):
    """Add multiple storage_pools."""
    return IMPL.storage_pools_create(context, storage_pools)


def storage_pool_update(context, storage_pool_id, storage_pool):
    """Update a storage_pool."""
    return IMPL.storage_pool_update(context, storage_pool_id, storage_pool)


def storage_pools_update(context, storage_pools):
    """Update multiple storage_pools"""
    return IMPL.storage_pools_update(context, storage_pools)


def storage_pools_delete(context, storage_pools):
    """Delete storage_pools."""
    return IMPL.storage_pools_delete(context, storage_pools)


def storage_pool_get(context, storage_pool_id):
    """Get a storage_pool or raise an exception if it does not exist."""
    return IMPL.storage_pool_get(context, storage_pool_id)


def storage_pool_get_all(context, marker=None, limit=None, sort_keys=None,
                         sort_dirs=None, filters=None, offset=None):
    """Retrieves all  storage_pools.

    If no sort parameters are specified then the returned volumes are sorted
    first by the 'created_at' key and then by the 'id' key in descending
    order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of  storage_pools
    """
    return IMPL.storage_pool_get_all(context, marker, limit,
                                     sort_keys, sort_dirs, filters, offset)


def storage_pool_delete_by_storage(context, storage_id):
    """Delete all the storage_pool of a device."""
    return IMPL.storage_pool_delete_by_storage(context, storage_id)


def disk_create(context, values):
    """Create a disk from the values dictionary."""
    return IMPL.disk_create(context, values)


def disk_update(context, disk_id, values):
    """Update a disk withe the values dictionary."""
    return IMPL.disk_create(context, disk_id, values)


def disk_get(context, disk_id):
    """Get a disk or raise an exception if it does not exist."""
    return IMPL.disk_get(context, disk_id)


def disk_get_all(context, marker=None, limit=None, sort_keys=None,
                 sort_dirs=None, filters=None, offset=None):
    """Retrieves all disks.

    If no sort parameters are specified then the returned volumes are sorted
    first by the 'created_at' key and then by the 'id' key in descending
    order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of disks
    """
    return IMPL.disk_get_all(context, marker, limit, sort_keys, sort_dirs,
                             filters, offset)


def access_info_create(context, values):
    """Create a storage access information that used to connect
    to a specific storage device.
    """
    return IMPL.access_info_create(context, values)


def access_info_update(context, storage_id, values):
    """Update a storage access information with the values dictionary."""
    return IMPL.access_info_update(context, storage_id, values)


def access_info_get(context, storage_id):
    """Get a storage access information."""
    return IMPL.access_info_get(context, storage_id)


def access_info_delete(context, storage_id):
    """Delete a storage access information."""
    return IMPL.access_info_delete(context, storage_id)


def access_info_get_all(context, marker=None, limit=None, sort_keys=None,
                        sort_dirs=None, filters=None, offset=None):
    """Retrieves all storage access information.

    If no sort parameters are specified then the returned volumes are sorted
    first by the 'created_at' key and then by the 'id' key in descending
    order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of storage accesses
    """
    return IMPL.access_info_get_all(context, marker, limit,
                                    sort_keys, sort_dirs, filters, offset)


def is_orm_value(obj):
    """Check if object is an ORM field."""
    return IMPL.is_orm_value(obj)


def alert_source_create(context, values):
    """Create an alert source."""
    return IMPL.alert_source_create(context, values)


def alert_source_update(context, storage_id, values):
    """Update an alert source."""
    return IMPL.alert_source_update(context, storage_id, values)


def alert_source_get(context, storage_id):
    """Get an alert source."""
    return IMPL.alert_source_get(context, storage_id)


def alert_source_delete(context, storage_id):
    """Delete an alert source."""
    return IMPL.alert_source_delete(context, storage_id)


def alert_source_get_all(context, marker=None, limit=None, sort_keys=None,
                         sort_dirs=None, filters=None, offset=None):
    """Retrieves all alert sources.

    If no sort parameters are specified then the returned alert sources are
    sorted first by the 'created_at' key in descending order.

    :param context: context of this request, it's helpful to trace the request
    :param marker: the last item of the previous page, used to determine the
                   next page of results to return
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys, for example
                      'desc' for descending order
    :param filters: dictionary of filters
    :param offset: number of items to skip
    :returns: list of storage accesses
    """
    return IMPL.alert_source_get_all(context, marker, limit, sort_keys,
                                     sort_dirs, filters, offset)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin.api.validation import parameter_types

# engineId is in range (5-32) octet which is 10-64 hex characters
# If it is odd length, 0 will be prefixed to last octet, so minimum length is 9
put = {
    'type': 'object',
    'properties': {
        'host': parameter_types.hostname_or_ip_address,
        'version': parameter_types.snmp_version,
        'community_string': {'type': 'string',
                             'minLength': 1,
                             'maxLength': 32},
        'username': {'type': 'string', 'minLength': 1, 'maxLength': 32},
        'security_level': parameter_types.snmp_security_level,
        'auth_key': {'type': 'string', 'minLength': 8, 'maxLength': 65535},
        'auth_protocol': parameter_types.snmp_auth_protocol,
        'privacy_protocol': parameter_types.snmp_privacy_protocol,
        'privacy_key': {'type': 'string', 'minLength': 8, 'maxLength': 65535},
        'engine_id': {'type': 'string', 'minLength': 9, 'maxLength': 64},
        'context_name': {'type': 'string', 'minLength': 0, 'maxLength': 32},
        'retry_num': {'type': 'integer'},
        'expiration': {'type': 'integer'},
        'port': parameter_types.tcp_udp_port
    },
    'required': ['host', 'version'],
    'additionalProperties': False,
}



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import paramiko as paramiko
from oslo_log import log as logging
from paramiko.hostkeys import HostKeyEntry

from delfin import exception

LOG = logging.getLogger(__name__)


class SSHClient(object):
    SOCKET_TIMEOUT = 10

    def __init__(self, **kwargs):

        ssh_access = kwargs.get('ssh')
        if ssh_access is None:
            raise exception.InvalidInput('Input ssh_access is missing')
        self.ssh_host = ssh_access.get('host')
        self.ssh_port = ssh_access.get('port')
        self.ssh_username = ssh_access.get('username')
        self.ssh_password = ssh_access.get('password')
        self.ssh_pub_key_type = ssh_access.get('pub_key_type')
        self.ssh_pub_key = ssh_access.get('pub_key')
        self.ssh_conn_timeout = ssh_access.get('conn_timeout')
        if self.ssh_conn_timeout is None:
            self.ssh_conn_timeout = SSHClient.SOCKET_TIMEOUT

    def connect(self):
        self.ssh = paramiko.SSHClient()

        if self.ssh_pub_key is None:
            self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        else:
            host_key = (self.ssh_host + ' ' + self.ssh_pub_key_type
                        + ' ' + self.ssh_pub_key)
            self.sethostkey(host_key)

        self.ssh.connect(hostname=self.ssh_host, port=self.ssh_port,
                         username=self.ssh_username,
                         password=self.ssh_password,
                         timeout=self.ssh_conn_timeout)

    def sethostkey(self, host_key):
        """
        Set public key,because input kwargs parameter host_key is string,
        not a file path,we can not use load file to get public key,so we set
        it as a string.
        :param str host_key: the public key which as a string
        """
        if (len(host_key) == 0) or (host_key[0] == "#"):
            return
        try:
            e = HostKeyEntry.from_line(host_key)
        except exception.SSHException:
            return
        if e is not None:
            _hostnames = e.hostnames
            for h in _hostnames:
                if self.ssh._host_keys.check(h, e.key):
                    e.hostnames.remove(h)
            if len(e.hostnames):
                self.ssh._host_keys._entries.append(e)

    def execCommand(self, command_str):
        result = None
        try:
            if command_str is not None:
                if self.ssh is not None:
                    stdin, stdout, stderr = self.ssh.exec_command(command_str)
                    res, err = stdout.read(), stderr.read()
                    re = res if res else err
                    result = re.decode()
        except Exception as e:
            LOG.error(e)
            result = e
        return result

    def close(self):
        try:
            if self.ssh is not None:
                # Close connection
                self.ssh.close()
                self.ssh = None
        except Exception as e:
            LOG.error(e)

    def doexec(self, context, command_str):
        """Execute command"""
        re = None
        try:
            if command_str is not None:
                self.connect()
                re = self.execCommand(command_str)
        except paramiko.AuthenticationException as ae:
            LOG.error('doexec Authentication error:{}'.format(ae))
            raise exception.InvalidUsernameOrPassword()
        except Exception as e:
            LOG.error('doexec InvalidUsernameOrPassword error:{}'.format(e))
            if 'WSAETIMEDOUT' in str(e):
                raise exception.SSHConnectTimeout()
            elif 'No authentication methods available' in str(e) \
                    or 'Authentication failed' in str(e):
                raise exception.SSHInvalidUsernameOrPassword()
            elif 'not a valid RSA private key file' in str(e):
                raise exception.InvalidPrivateKey()
            elif 'not found in known_hosts' in str(e):
                raise exception.SSHNotFoundKnownHosts(self.ssh_host)
            else:
                raise exception.SSHException()

        finally:
            self.close()
        return re



# Copyright 2020 The SODA Authors.
# Copyright (c) 2020 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import sys

from oslo_log import log as logging
import requests
import urllib3
import requests.auth
import requests.exceptions as r_exc
import six

from delfin import exception
from delfin.i18n import _

LOG = logging.getLogger(__name__)
SLOPROVISIONING = 'sloprovisioning'
U4V_VERSION = '92'
UCODE_5978 = '5978'
# HTTP constants
GET = 'GET'
POST = 'POST'
PUT = 'PUT'
DELETE = 'DELETE'
STATUS_200 = 200
STATUS_201 = 201
STATUS_202 = 202
STATUS_204 = 204
STATUS_401 = 401

# Default expiration time(in sec) for vmax connect request
VERSION_GET_TIME_OUT = 10


class VMaxRest(object):
    """Rest class based on Unisphere for VMax Rest API."""

    def __init__(self):
        self.session = None
        self.base_uri = None
        self.user = None
        self.passwd = None
        self.verify = None
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    def set_rest_credentials(self, array_info):
        """Given the array record set the rest server credentials.
        :param array_info: record
        """
        ip = array_info['host']
        port = array_info['port']
        self.user = array_info['username']
        self.passwd = array_info['password']
        ip_port = "%(ip)s:%(port)d" % {'ip': ip, 'port': port}
        self.base_uri = ("https://%(ip_port)s/univmax/restapi" % {
            'ip_port': ip_port})

    def establish_rest_session(self):
        """Establish the rest session.
        :returns: requests.session() -- session, the rest session
        """
        LOG.info("Establishing REST session with %(base_uri)s",
                 {'base_uri': self.base_uri})
        if self.session:
            self.session.close()
        session = requests.session()
        session.headers = {'content-type': 'application/json',
                           'accept': 'application/json',
                           'Application-Type': 'delfin'}
        session.auth = requests.auth.HTTPBasicAuth(self.user, self.passwd)

        if not self.verify:
            session.verify = False
        else:
            LOG.debug("Enable certificate verification, ca_path: {0}".format(
                self.verify))
            session.verify = self.verify

        return session

    def request(self, target_uri, method, params=None, request_object=None,
                timeout=None):
        """Sends a request (GET, POST, PUT, DELETE) to the target api.
        :param target_uri: target uri (string)
        :param method: The method (GET, POST, PUT, or DELETE)
        :param params: Additional URL parameters
        :param request_object: request payload (dict)
        :param timeout: expiration timeout(in sec)
        :returns: server response object (dict)
        :raises: StorageBackendException, Timeout, ConnectionError,
                 HTTPError, SSLError
        """

        url, message, status_code, response = None, None, None, None
        if not self.session:
            self.session = self.establish_rest_session()

        try:
            url = ("%(self.base_uri)s%(target_uri)s" % {
                'self.base_uri': self.base_uri,
                'target_uri': target_uri})

            if request_object:
                response = self.session.request(
                    method=method, url=url,
                    data=json.dumps(request_object, sort_keys=True,
                                    indent=4), timeout=timeout)
            elif params:
                response = self.session.request(
                    method=method, url=url, params=params, timeout=timeout)
            else:
                response = self.session.request(
                    method=method, url=url, timeout=timeout)

            status_code = response.status_code

            try:
                message = response.json()
            except ValueError:
                LOG.debug("No response received from API. Status code "
                          "received is: %(status_code)s", {
                              'status_code': status_code})
                message = None

            LOG.debug("%(method)s request to %(url)s has returned with "
                      "a status code of: %(status_code)s.", {
                          'method': method, 'url': url,
                          'status_code': status_code})

        except r_exc.SSLError as e:
            msg = _("The connection to %(base_uri)s has encountered an "
                    "SSL error. Please check your SSL config or supplied "
                    "SSL cert in Delfin configuration. SSL Exception "
                    "message: %(e)s")
            raise r_exc.SSLError(msg, {'base_uri': self.base_uri, 'e': e})

        except (r_exc.Timeout, r_exc.ConnectionError,
                r_exc.HTTPError) as e:
            exc_class, __, __ = sys.exc_info()
            msg = _("The %(method)s to Unisphere server %(base)s has "
                    "experienced a %(error)s error. Please check your "
                    "Unisphere server connection/availability. "
                    "Exception message: %(exc_msg)s")
            raise exc_class(msg % {'method': method,
                                   'base': self.base_uri,
                                   'error': e.__class__.__name__,
                                   'exc_msg': e})

        except Exception as e:
            msg = _("The %(method)s request to URL %(url)s failed with "
                    "exception %(e)s")
            LOG.error(msg, {'method': method, 'url': url,
                            'e': six.text_type(e)})
            raise exception.StorageBackendException(
                message=(msg, {'method': method, 'url': url,
                               'e': six.text_type(e)}))

        return status_code, message

    @staticmethod
    def check_status_code_success(operation, status_code, message):
        """Check if a status code indicates success.
        :param operation: the operation
        :param status_code: the status code
        :param message: the server response
        :raises: StorageBackendException
        """
        if status_code not in [STATUS_200, STATUS_201,
                               STATUS_202, STATUS_204]:
            exception_message = (
                _("Error %(operation)s. The status code received is %(sc)s "
                  "and the message is %(message)s.") % {
                    'operation': operation, 'sc': status_code,
                    'message': message})
            raise exception.StorageBackendException(
                message=exception_message)

    def build_uri(self, *args, **kwargs):
        """Build the target url.
        :param args: input args, see _build_uri_legacy_args() for input
                     breakdown
        :param kwargs: input keyword args, see _build_uri_kwargs() for input
                       breakdown
        :return: target uri -- str
        """
        if args:
            target_uri = self._build_uri_legacy_args(*args, **kwargs)
        else:
            target_uri = self._build_uri_kwargs(**kwargs)

        return target_uri

    @staticmethod
    def _build_uri_legacy_args(*args, **kwargs):
        """Build the target URI using legacy args & kwargs.
        Expected format:
            arg[0]: the array serial number: the array serial number -- str
            arg[1]: the resource category e.g. 'sloprovisioning' -- str
            arg[2]: the resource type e.g. 'maskingview' -- str
            kwarg resource_name: the name of a specific resource -- str
            kwarg private: if endpoint is private -- bool
            kwarg version: U4V REST endpoint version -- int/str
            kwarg no_version: if endpoint should be versionless -- bool
        :param args: input args -- see above
        :param kwargs: input keyword args -- see above
        :return: target URI -- str
        """
        # Extract args following legacy _build_uri() format
        array_id, category, resource_type = args[0], args[1], args[2]
        # Extract keyword args following legacy _build_uri() format
        resource_name = kwargs.get('resource_name')
        private = kwargs.get('private')
        version = kwargs.get('version', U4V_VERSION)
        if kwargs.get('no_version'):
            version = None

        # Build URI
        target_uri = ''
        if private:
            target_uri += '/private'
        if version:
            target_uri += '/%(version)s' % {'version': version}
        target_uri += (
            '/{cat}/symmetrix/{array_id}/{res_type}'.format(
                cat=category, array_id=array_id, res_type=resource_type))
        if resource_name:
            target_uri += '/{resource_name}'.format(
                resource_name=kwargs.get('resource_name'))

        return target_uri

    @staticmethod
    def _build_uri_kwargs(**kwargs):
        """Build the target URI using kwargs.
        Expected kwargs:
            private: if endpoint is private (optional) -- bool
            version: U4P REST endpoint version (optional) -- int/None
            no_version: if endpoint should be versionless (optional) -- bool
            category: U4P REST category eg. 'common', 'replication'-- str
            resource_level: U4P REST resource level eg. 'symmetrix'
                            (optional) -- str
            resource_level_id: U4P REST resource level id (optional) -- str
            resource_type: U4P REST resource type eg. 'rdf_director', 'host'
                           (optional) -- str
            resource_type_id: U4P REST resource type id (optional) -- str
            resource: U4P REST resource eg. 'port' (optional) -- str
            resource_id: U4P REST resource id (optional) -- str
            object_type: U4P REST resource eg. 'rdf_group' (optional) -- str
            object_type_id: U4P REST resource id (optional) -- str
        :param kwargs: input keyword args -- see above
        :return: target URI -- str
        """
        version = kwargs.get('version', U4V_VERSION)
        if kwargs.get('no_version'):
            version = None

        target_uri = ''

        if kwargs.get('private'):
            target_uri += '/private'

        if version:
            target_uri += '/%(ver)s' % {'ver': version}

        target_uri += '/%(cat)s' % {'cat': kwargs.get('category')}

        if kwargs.get('resource_level'):
            target_uri += '/%(res_level)s' % {
                'res_level': kwargs.get('resource_level')}

        if kwargs.get('resource_level_id'):
            target_uri += '/%(res_level_id)s' % {
                'res_level_id': kwargs.get('resource_level_id')}

        if kwargs.get('resource_type'):
            target_uri += '/%(res_type)s' % {
                'res_type': kwargs.get('resource_type')}
            if kwargs.get('resource_type_id'):
                target_uri += '/%(res_type_id)s' % {
                    'res_type_id': kwargs.get('resource_type_id')}

        if kwargs.get('resource'):
            target_uri += '/%(res)s' % {
                'res': kwargs.get('resource')}
            if kwargs.get('resource_id'):
                target_uri += '/%(res_id)s' % {
                    'res_id': kwargs.get('resource_id')}

        if kwargs.get('object_type'):
            target_uri += '/%(object_type)s' % {
                'object_type': kwargs.get('object_type')}
            if kwargs.get('object_type_id'):
                target_uri += '/%(object_type_id)s' % {
                    'object_type_id': kwargs.get('object_type_id')}

        return target_uri

    def get_request(self, target_uri, resource_type, params=None):
        """Send a GET request to the array.
        :param target_uri: the target uri
        :param resource_type: the resource type, e.g. maskingview
        :param params: optional dict of filter params
        :returns: resource_object -- dict or None
        """
        resource_object = None
        sc, message = self.request(target_uri, GET, params=params)
        operation = 'get %(res)s' % {'res': resource_type}
        try:
            self.check_status_code_success(operation, sc, message)
        except Exception as e:
            LOG.debug("Get resource failed with %(e)s",
                      {'e': e})
        if sc == STATUS_200:
            resource_object = message
            resource_object = self.list_pagination(resource_object)
        return resource_object

    def get_alert_request(self, target_uri):
        """Send a GET request to the array.
        :param target_uri: the target uri
        :returns: resource_object -- dict or None
        """
        sc, message = self.request(target_uri, GET, params=None)
        if sc != STATUS_200:
            raise exception.StorageListAlertFailed(message)
        resource_object = message
        resource_object = self.list_pagination(resource_object)
        return resource_object

    def get_resource(self, array, category, resource_type,
                     resource_name=None, params=None, private=False,
                     version=U4V_VERSION):
        """Get resource details from array.
        :param array: the array serial number
        :param category: the resource category e.g. sloprovisioning
        :param resource_type: the resource type e.g. maskingview
        :param resource_name: the name of a specific resource
        :param params: query parameters
        :param private: empty string or '/private' if private url
        :param version: None or specific version number if required
        :returns: resource object -- dict or None
        """
        target_uri = self.build_uri(
            array, category, resource_type, resource_name=resource_name,
            private=private, version=version)
        return self.get_request(target_uri, resource_type, params)

    def get_array_detail(self, version=U4V_VERSION, array=''):
        """Get an array from its serial number.
        :param array: the array serial number
        :param version: the unisphere version
        :returns: array_details -- dict or None
        """
        target_uri = '/%s/system/symmetrix/%s' % (version, array)
        array_details = self.get_request(target_uri, 'system')
        if not array_details:
            LOG.error("Cannot connect to array %(array)s.",
                      {'array': array})
        return array_details

    def get_uni_version(self):
        """Get the unisphere version from the server.
        :returns: version and major_version(e.g. ("V8.4.0.16", "84"))
        """
        version, major_version = None, None
        response = self.get_unisphere_version()
        if response and response.get('version'):
            version = response['version']
            version_list = version.split('.')
            major_version = version_list[0][1] + version_list[1]
        return version, major_version

    def get_unisphere_version(self):
        """Get the unisphere version from the server.
        :returns: version dict
        """
        post_90_endpoint = '/version'
        pre_91_endpoint = '/system/version'

        status_code, version_dict = self.request(
            post_90_endpoint, GET, timeout=VERSION_GET_TIME_OUT)
        if status_code is not STATUS_200:
            status_code, version_dict = self.request(
                pre_91_endpoint, GET, timeout=VERSION_GET_TIME_OUT)

        if status_code == STATUS_401:
            raise exception.InvalidCredential()

        if not version_dict:
            LOG.error("Unisphere version info not found.")
        return version_dict

    def get_srp_by_name(self, array, version, srp=None):
        """Returns the details of a storage pool.
        :param array: the array serial number
        :param version: the unisphere version
        :param srp: the storage resource pool name
        :returns: SRP_details -- dict or None
        """
        LOG.debug("storagePoolName: %(srp)s, array: %(array)s.",
                  {'srp': srp, 'array': array})
        srp_details = self.get_resource(array, SLOPROVISIONING, 'srp',
                                        resource_name=srp, version=version,
                                        params=None)
        return srp_details

    def get_vmax_array_details(self, version=U4V_VERSION, array=''):
        """Get the VMax array properties.
        :param version: the unisphere version
        :param array: the array serial number
        :returns: the VMax model
        """
        system_info = self.get_array_detail(version, array)
        vmax_version = system_info.get('model')
        vmax_ucode = system_info.get('ucode')
        vmax_display_name = system_info.get('display_name')
        array_details = {"model": vmax_version,
                         "ucode": vmax_ucode,
                         "display_name": vmax_display_name}
        return array_details

    def get_array_model_info(self, version=U4V_VERSION, array=''):
        """Get the VMax model.
        :param version: the unisphere version
        :param array: the array serial number
        :returns: the VMax model
        """
        is_next_gen = False
        system_info = self.get_array_detail(version, array)
        array_model = system_info.get('model', None)
        ucode_version = system_info['ucode'].split('.')[0]
        if ucode_version >= UCODE_5978:
            is_next_gen = True
        return array_model, is_next_gen

    def get_storage_group(self, array, version, storage_group_name):
        """Given a name, return storage group details.
        :param version: the unisphere version
        :param array: the array serial number
        :param storage_group_name: the name of the storage group
        :returns: storage group dict or None
        """
        return self.get_resource(
            array, SLOPROVISIONING, 'storagegroup',
            version=version,
            resource_name=storage_group_name)

    def get_system_capacity(self, array, version):
        target_uri = '/%s/sloprovisioning/symmetrix/%s' % (version, array)
        capacity_details = self.get_request(target_uri, None)
        if not capacity_details:
            LOG.error("Cannot connect to array %(array)s.",
                      {'array': array})
        return capacity_details

    def get_default_srps(self, array, version=U4V_VERSION):
        """Get the VMax array default SRPs.
        :param version: the unisphere version
        :param array: the array serial number
        :returns: dictionary default SRPs
        """
        symmetrix_info = self.get_system_capacity(array, version)
        default_fba_srp = symmetrix_info.get('default_fba_srp', None)
        default_ckd_srp = symmetrix_info.get('default_ckd_srp', None)
        default_srps = {"FBA": default_fba_srp,
                        "CKD": default_ckd_srp}
        return default_srps

    def get_volume(self, array, version, device_id):
        """Get a VMax volume from array.
        :param array: the array serial number
        :param device_id: the volume device id
        :returns: volume dict
        :raises: StorageBackendException
        """
        volume_dict = self.get_resource(
            array, SLOPROVISIONING, 'volume', resource_name=device_id,
            version=version)
        if not volume_dict:
            exception_message = (_("Volume %(deviceID)s not found.")
                                 % {'deviceID': device_id})
            LOG.error(exception_message)
            raise exception.StorageBackendException(
                message=exception_message)
        return volume_dict

    def get_volume_list(self, array, version, params):
        """Get a filtered list of VMax volumes from array.
        Filter parameters are required as the unfiltered volume list could be
        very large and could affect performance if called often.
        :param array: the array serial number
        :param version: the unisphere version
        :param params: filter parameters
        :returns: device_ids -- list
        """
        device_ids = []
        volume_dict_list = self.get_resource(
            array, SLOPROVISIONING, 'volume', version=version, params=params)
        try:
            for vol_dict in volume_dict_list:
                device_id = vol_dict['volumeId']
                device_ids.append(device_id)
        except (KeyError, TypeError):
            pass
        return device_ids

    def list_pagination(self, list_info):
        """Process lists under or over the maxPageSize
        :param list_info: the object list information
        :returns: the result list
        """
        result_list = []
        try:
            result_list = list_info['resultList']['result']
            iterator_id = list_info['id']
            list_count = list_info['count']
            max_page_size = list_info['maxPageSize']
            start_position = list_info['resultList']['from']
            end_position = list_info['resultList']['to']
        except (KeyError, TypeError):
            return list_info
        if list_count > max_page_size:
            LOG.info("More entries exist in the result list, retrieving "
                     "remainder of results from iterator.")

            start_position = end_position + 1
            if list_count < (end_position + max_page_size):
                end_position = list_count
            else:
                end_position += max_page_size
            iterator_response = self.get_iterator_page_list(
                iterator_id, list_count, start_position, end_position,
                max_page_size)

            result_list += iterator_response
        return result_list

    def get_iterator_page_list(self, iterator_id, result_count, start_position,
                               end_position, max_page_size):
        """Iterate through response if more than one page available.
        :param iterator_id: the iterator ID
        :param result_count: the amount of results in the iterator
        :param start_position: position to begin iterator from
        :param end_position: position to stop iterator
        :param max_page_size: the max page size
        :returns: list -- merged results from multiple pages
        """
        iterator_result = []
        has_more_entries = True

        while has_more_entries:
            if start_position <= result_count <= end_position:
                end_position = result_count
                has_more_entries = False

            params = {'to': end_position, 'from': start_position}
            target_uri = ('/common/Iterator/%(iterator_id)s/page' % {
                'iterator_id': iterator_id})
            iterator_response = self.get_request(target_uri, 'iterator',
                                                 params)
            try:
                iterator_result += iterator_response['result']
                start_position += max_page_size
                end_position += max_page_size
            except (KeyError, TypeError):
                pass

        return iterator_result

    def get_alerts(self, array, version):
        """Get all alerts with given version and arrayid
        :param array: the array serial number
        :param version: the unisphere version
        :returns: alert_list -- dict or None
        """
        target_uri = '/%s/system/symmetrix/%s/alert?acknowledged=false' \
                     % (version, array)

        # First get list of all alert ids
        alert_id_list = self.get_alert_request(target_uri)
        if not alert_id_list:
            # No current alert ids found
            return []

        # For each alert id, get details of alert
        # Above list is prefixed with 'alertId'
        alert_id_list = alert_id_list['alertId']
        alert_list = []
        for alert_id in alert_id_list:
            target_uri = '/%s/system/symmetrix/%s/alert/%s' \
                         % (version, array, alert_id)
            alert = self.get_alert_request(target_uri)
            if alert is not None:
                alert_list.append(alert)

        return alert_list

    def clear_alert(self, sequence_number, array, version):
        """Clears alert for given sequence number
        :param sequence_number: unique id of the alert
        :param array: the array serial number
        :param version: the unisphere version
        :returns: result -- success/failure
        """
        target_uri = '/%s/system/symmetrix/%s/alert/%s' \
                     % (version, array, sequence_number)

        status, message = self.request(target_uri, DELETE, params=None)
        if status != STATUS_204:
            raise exception.StorageClearAlertFailed(message)
        return status



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from delfin.common import constants
from delfin.drivers.huawei.oceanstor import rest_client, consts, alert_handler
from delfin.drivers import driver
from delfin import exception

LOG = log.getLogger(__name__)


class OceanStorDriver(driver.StorageDriver):
    """OceanStorDriver implement Huawei OceanStor driver,
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.client = rest_client.RestClient(**kwargs)
        self.sector_size = consts.SECTORS_SIZE

    def reset_connection(self, context, **kwargs):
        self.client.reset_connection(**kwargs)

    def get_storage(self, context):

        storage = self.client.get_storage()

        # Get firmware version
        controller = self.client.get_controller()
        firmware_ver = controller[0]['SOFTVER']

        # Get status
        status = constants.StorageStatus.OFFLINE
        if storage['RUNNINGSTATUS'] == consts.STATUS_STORAGE_NORMAL:
            status = constants.StorageStatus.NORMAL

        # Keep sector_size for use in list pools
        self.sector_size = int(storage['SECTORSIZE'])

        total_cap = int(storage['TOTALCAPACITY']) * self.sector_size
        used_cap = int(storage['USEDCAPACITY']) * self.sector_size
        free_cap = int(storage['userFreeCapacity']) * self.sector_size
        raw_cap = int(storage['MEMBERDISKSCAPACITY']) * self.sector_size

        s = {
            'name': 'OceanStor',
            'vendor': 'Huawei',
            'description': 'Huawei OceanStor Storage',
            'model': storage['NAME'],
            'status': status,
            'serial_number': storage['ID'],
            'firmware_version': firmware_ver,
            'location': storage['LOCATION'],
            'total_capacity': total_cap,
            'used_capacity': used_cap,
            'free_capacity': free_cap,
            'raw_capacity': raw_cap
        }
        LOG.info("get_storage(), successfully retrieved storage details")
        return s

    def list_storage_pools(self, context):
        try:
            # Get list of OceanStor pool details
            pools = self.client.get_all_pools()

            pool_list = []
            for pool in pools:
                # Get pool status
                status = constants.StoragePoolStatus.OFFLINE
                if pool['RUNNINGSTATUS'] == consts.STATUS_POOL_ONLINE:
                    status = constants.StoragePoolStatus.NORMAL

                # Get pool storage_type
                storage_type = constants.StorageType.BLOCK
                if pool.get('USAGETYPE') == consts.FILE_SYSTEM_POOL_TYPE:
                    storage_type = constants.StorageType.FILE

                total_cap = \
                    int(pool['USERTOTALCAPACITY']) * self.sector_size
                used_cap = \
                    int(pool['USERCONSUMEDCAPACITY']) * self.sector_size
                free_cap = \
                    int(pool['USERFREECAPACITY']) * self.sector_size

                p = {
                    'name': pool['NAME'],
                    'storage_id': self.storage_id,
                    'native_storage_pool_id': pool['ID'],
                    'description': 'Huawei OceanStor Pool',
                    'status': status,
                    'storage_type': storage_type,
                    'total_capacity': total_cap,
                    'used_capacity': used_cap,
                    'free_capacity': free_cap,
                }
                pool_list.append(p)

            return pool_list

        except Exception as err:
            LOG.error(
                "Failed to get pool metrics from OceanStor: {}".format(err))
            raise exception.StorageBackendException(
                reason='Failed to get pool metrics from OceanStor')

    def list_volumes(self, context):
        try:
            # Get all volumes in OceanStor
            volumes = self.client.get_all_volumes()
            pools = self.client.get_all_pools()

            volume_list = []
            for volume in volumes:
                # Get pool id of volume
                orig_pool_id = ''
                for pool in pools:
                    if volume['PARENTNAME'] == pool['NAME']:
                        orig_pool_id = pool['ID']

                compressed = False
                if volume['ENABLECOMPRESSION'] != 'false':
                    compressed = True

                deduplicated = False
                if volume['ENABLEDEDUP'] != 'false':
                    deduplicated = True

                status = constants.VolumeStatus.ERROR
                if volume['RUNNINGSTATUS'] == consts.STATUS_VOLUME_READY:
                    status = constants.VolumeStatus.AVAILABLE

                vol_type = constants.VolumeType.THICK
                if volume['ALLOCTYPE'] == consts.THIN_LUNTYPE:
                    vol_type = constants.VolumeType.THIN

                sector_size = int(volume['SECTORSIZE'])
                total_cap = int(volume['CAPACITY']) * sector_size
                used_cap = int(volume['ALLOCCAPACITY']) * sector_size

                v = {
                    'name': volume['NAME'],
                    'storage_id': self.storage_id,
                    'description': 'Huawei OceanStor volume',
                    'status': status,
                    'native_volume_id': volume['ID'],
                    'native_storage_pool_id': orig_pool_id,
                    'wwn': volume['WWN'],
                    'type': vol_type,
                    'total_capacity': total_cap,
                    'used_capacity': used_cap,
                    'free_capacity': None,
                    'compressed': compressed,
                    'deduplicated': deduplicated,
                }

                volume_list.append(v)

            return volume_list

        except Exception as err:
            LOG.error(
                "Failed to get list volumes from OceanStor: {}".format(err))
            raise exception.StorageBackendException(
                reason='Failed to get list volumes from OceanStor')

    def add_trap_config(self, context, trap_config):
        pass

    def remove_trap_config(self, context, trap_config):
        pass

    def parse_alert(self, context, alert):
        return alert_handler.AlertHandler().parse_alert(context, alert)

    def clear_alert(self, context, sequence_number):
        return self.client.clear_alert(sequence_number)

    def list_alerts(self, context):
        # First query alerts and then translate to model
        alert_list = self.client.list_alerts()
        alert_model_list = alert_handler.AlertHandler()\
            .parse_queried_alerts(alert_list)
        return alert_model_list



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import TestCase, mock
from delfin import exception
from delfin import context
from delfin.drivers.dell_emc.vmax.vmax import VMAXStorageDriver
from delfin.drivers.dell_emc.vmax.rest import VMaxRest


class Request:
    def __init__(self):
        self.environ = {'delfin.context': context.RequestContext()}
        pass


VMAX_STORAGE_CONF = {
    "storage_id": "12345",
    "vendor": "dell_emc",
    "model": "vmax",
    "rest": {
        "host": "10.0.0.1",
        "port": "8443",
        "username": "user",
        "password": "pass"
    },
    "extra_attributes": {
        "array_id": "00112233"
    }
}


class TestVMAXStorageDriver(TestCase):

    @mock.patch.object(VMaxRest, 'get_array_detail')
    @mock.patch.object(VMaxRest, 'get_uni_version')
    @mock.patch.object(VMaxRest, 'set_rest_credentials')
    def test_init(self,
                  mock_rest, mock_version, mock_array):
        kwargs = VMAX_STORAGE_CONF

        mock_rest.return_value = None
        mock_version.return_value = ['V9.0.2.7', '90']
        mock_array.return_value = {'symmetrixId': ['00112233']}

        driver = VMAXStorageDriver(**kwargs)
        self.assertEqual(driver.client.uni_version, '90')
        self.assertEqual(driver.storage_id, "12345")
        self.assertEqual(driver.client.array_id, "00112233")

        with self.assertRaises(Exception) as exc:
            mock_rest.side_effect = exception.StorageBackendException
            VMAXStorageDriver(**kwargs)
        self.assertIn('Exception from Storage Backend', str(exc.exception))

        with self.assertRaises(Exception) as exc:
            mock_rest.side_effect = None
            mock_version.side_effect = exception.StorageBackendException
            VMAXStorageDriver(**kwargs)
        self.assertIn('Exception from Storage Backend', str(exc.exception))

        with self.assertRaises(Exception) as exc:
            mock_rest.side_effect = None
            mock_version.side_effect = ['V9.0.2.7', '90']
            mock_array.side_effect = exception.StorageBackendException
            VMAXStorageDriver(**kwargs)
        self.assertIn('Failed to connect to VMAX', str(exc.exception))

    @mock.patch.object(VMaxRest, 'get_system_capacity')
    @mock.patch.object(VMaxRest, 'get_vmax_array_details')
    @mock.patch.object(VMaxRest, 'get_array_detail')
    @mock.patch.object(VMaxRest, 'get_uni_version')
    @mock.patch.object(VMaxRest, 'set_rest_credentials')
    def test_get_storage(self,
                         mock_rest, mock_version, mock_array,
                         mock_array_details, mock_capacity):
        expected = {
            'name': 'VMAX250F-00112233',
            'vendor': 'Dell EMC',
            'description': '',
            'model': 'VMAX250F',
            'firmware_version': '5978.221.221',
            'status': 'normal',
            'serial_number': '00112233',
            'location': '',
            'total_capacity': 109951162777600,
            'used_capacity': 82463372083200,
            'free_capacity': 27487790694400,
            'raw_capacity': 1610612736000,
            'subscribed_capacity': 219902325555200
        }
        system_capacity = {
            'system_capacity': {
                'usable_total_tb': 100,
                'usable_used_tb': 75,
                'subscribed_total_tb': 200
            },
            'physicalCapacity': {
                'total_capacity_gb': 1500

            }
        }
        kwargs = VMAX_STORAGE_CONF

        mock_rest.return_value = None
        mock_version.return_value = ['V9.0.2.7', '90']
        mock_array.return_value = {'symmetrixId': ['00112233']}
        mock_array_details.return_value = {
            'model': 'VMAX250F',
            'ucode': '5978.221.221',
            'display_name': 'VMAX250F-00112233'}
        mock_capacity.return_value = system_capacity

        driver = VMAXStorageDriver(**kwargs)

        self.assertEqual(driver.storage_id, "12345")
        self.assertEqual(driver.client.array_id, "00112233")

        ret = driver.get_storage(context)
        self.assertDictEqual(ret, expected)

        mock_array_details.side_effect = exception.StorageBackendException
        with self.assertRaises(Exception) as exc:
            driver.get_storage(context)

        self.assertIn('Failed to get array details from VMAX',
                      str(exc.exception))

        mock_array_details.side_effect = [{
            'model': 'VMAX250F',
            'ucode': '5978.221.221',
            'display_name': 'VMAX250F-00112233'}]

        mock_capacity.side_effect = exception.StorageBackendException
        with self.assertRaises(Exception) as exc:
            driver.get_storage(context)

        self.assertIn('Failed to get capacity from VMAX',
                      str(exc.exception))

    @mock.patch.object(VMaxRest, 'get_srp_by_name')
    @mock.patch.object(VMaxRest, 'get_array_detail')
    @mock.patch.object(VMaxRest, 'get_uni_version')
    @mock.patch.object(VMaxRest, 'set_rest_credentials')
    def test_list_storage_pools(self,
                                mock_rest, mock_version,
                                mock_array, mock_srp):
        expected = [{
            'name': 'SRP_1',
            'storage_id': '12345',
            'native_storage_pool_id': 'SRP_ID',
            'description': 'Dell EMC VMAX Pool',
            'status': 'normal',
            'storage_type': 'block',
            'total_capacity': 109951162777600,
            'used_capacity': 82463372083200,
            'free_capacity': 27487790694400,
            'subscribed_capacity': 219902325555200
        }]
        pool_info = {
            'srp_capacity': {
                'usable_total_tb': 100,
                'usable_used_tb': 75,
                'subscribed_total_tb': 200
            },
            'srpId': 'SRP_ID'
        }
        kwargs = VMAX_STORAGE_CONF
        mock_rest.return_value = None
        mock_version.return_value = ['V9.0.2.7', '90']
        mock_array.return_value = {'symmetrixId': ['00112233']}
        mock_srp.side_effect = [{'srpId': ['SRP_1']}, pool_info]

        driver = VMAXStorageDriver(**kwargs)
        self.assertEqual(driver.storage_id, "12345")
        self.assertEqual(driver.client.array_id, "00112233")

        ret = driver.list_storage_pools(context)
        self.assertDictEqual(ret[0], expected[0])

        mock_srp.side_effect = [{'srpId': ['SRP_1']},
                                exception.StorageBackendException]
        with self.assertRaises(Exception) as exc:
            driver.list_storage_pools(context)

        self.assertIn('Failed to get pool metrics from VMAX',
                      str(exc.exception))

        mock_srp.side_effect = [exception.StorageBackendException, pool_info]
        with self.assertRaises(Exception) as exc:
            driver.list_storage_pools(context)

        self.assertIn('Failed to get pool metrics from VMAX',
                      str(exc.exception))

    @mock.patch.object(VMaxRest, 'get_system_capacity')
    @mock.patch.object(VMaxRest, 'get_storage_group')
    @mock.patch.object(VMaxRest, 'get_volume')
    @mock.patch.object(VMaxRest, 'get_volume_list')
    @mock.patch.object(VMaxRest, 'get_array_detail')
    @mock.patch.object(VMaxRest, 'get_uni_version')
    @mock.patch.object(VMaxRest, 'set_rest_credentials')
    def test_list_volumes(self,
                          mock_rest, mock_version, mock_array,
                          mock_vols, mock_vol, mock_sg, mock_capacity):
        expected = [{
            'name': 'volume_1',
            'storage_id': '12345',
            'description': "Dell EMC VMAX 'thin device' volume",
            'type': 'thin',
            'status': 'available',
            'native_volume_id': '00001',
            'wwn': 'wwn123',
            'total_capacity': 104857600,
            'used_capacity': 10485760,
            'free_capacity': 94371840,
            'native_storage_pool_id': 'SRP_1',
            'compressed': True
        }]
        volumes = {
            'volumeId': '00001',
            'cap_mb': 100,
            'allocated_percent': 10,
            'status': 'Ready',
            'type': 'TDEV',
            'wwn': 'wwn123',
            'num_of_storage_groups': 1,
            'storageGroupId': ['SG_001'],
            'emulation': 'FBA'
        }
        volumes1 = {
            'volumeId': '00002',
            'cap_mb': 100,
            'allocated_percent': 10,
            'status': 'Ready',
            'type': 'TDEV',
            'wwn': 'wwn1234',
            'num_of_storage_groups': 0,
            'storageGroupId': [],
            'emulation': 'FBA'
        }
        volumes2 = {
            'volumeId': '00003',
            'cap_mb': 100,
            'allocated_percent': 10,
            'status': 'Ready',
            'type': 'TDEV',
            'wwn': 'wwn1234',
            'num_of_storage_groups': 0,
            'storageGroupId': [],
            'emulation': 'CKD'
        }
        storage_group_info = {
            'srp': 'SRP_1',
            'compression': True
        }
        default_srps = {
            'default_fba_srp': 'SRP_1',
            'default_ckd_srp': 'SRP_2'
        }
        kwargs = VMAX_STORAGE_CONF
        mock_rest.return_value = None
        mock_version.return_value = ['V9.0.2.7', '90']
        mock_array.return_value = {'symmetrixId': ['00112233']}
        mock_vols.side_effect = [['volume_1', 'volume_2', 'volume_3']]
        mock_vol.side_effect = [volumes, volumes1, volumes2]
        mock_sg.side_effect = [storage_group_info]
        mock_capacity.return_value = default_srps

        driver = VMAXStorageDriver(**kwargs)
        self.assertEqual(driver.storage_id, "12345")
        self.assertEqual(driver.client.array_id, "00112233")
        ret = driver.list_volumes(context)
        self.assertDictEqual(ret[0], expected[0])

        mock_vols.side_effect = [['volume_1']]
        mock_vol.side_effect = [volumes]
        mock_sg.side_effect = [exception.StorageBackendException]
        with self.assertRaises(Exception) as exc:
            driver.list_volumes(context)

        self.assertIn('Failed to get list volumes from VMAX',
                      str(exc.exception))

        mock_vols.side_effect = [['volume_1']]
        mock_vol.side_effect = [exception.StorageBackendException]
        mock_sg.side_effect = [storage_group_info]
        with self.assertRaises(Exception) as exc:
            driver.list_volumes(context)

        self.assertIn('Failed to get list volumes from VMAX',
                      str(exc.exception))

        mock_vols.side_effect = [exception.StorageBackendException]
        mock_vol.side_effect = [volumes]
        mock_sg.side_effect = [storage_group_info]
        with self.assertRaises(Exception) as exc:
            driver.list_volumes(context)

        self.assertIn('Failed to get list volumes from VMAX',
                      str(exc.exception))



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class OidMapper(object):
    """Functions/attributes for oid to alert info mapper"""

    # Map to translate trap oid strings to oid names
    OID_MAP = {"1.3.6.1.3.94.1.11.1.3": "connUnitEventId",
               "1.3.6.1.3.94.1.11.1.6": "connUnitEventSeverity",
               "1.3.6.1.3.94.1.11.1.7": "connUnitEventType",
               "1.3.6.1.3.94.1.11.1.8": "connUnitEventObject",
               "1.3.6.1.3.94.1.11.1.9": "connUnitEventDescr",
               "1.3.6.1.3.94.1.6.1.20": "connUnitName",
               "1.3.6.1.3.94.1.6.1.3": "connUnitType",
               "1.3.6.1.4.1.1139.3.8888.1.0": "emcAsyncEventSource",
               "1.3.6.1.4.1.1139.3.8888.2.0": "emcAsyncEventCode",
               "1.3.6.1.4.1.1139.3.8888.3.0": "emcAsyncEventComponentType",
               "1.3.6.1.4.1.1139.3.8888.4.0": "emcAsyncEventComponentName"}

    def __init__(self):
        pass

    def map_oids(self, alert):
        """Translate oids using static map."""
        alert_model = dict()

        for attr in alert:
            # Remove the instance number at the end of oid before mapping
            oid_str = attr.rsplit('.', 1)[0]
            key = self.OID_MAP.get(oid_str, None)
            alert_model[key] = alert[attr]

        return alert_model



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random

import decorator
import math
import six
import time
from oslo_config import cfg
from oslo_log import log
from oslo_utils import uuidutils

from delfin import exception
from delfin.drivers import driver

CONF = cfg.CONF

fake_opts = [
    cfg.StrOpt('fake_pool_range',
               default='1-100',
               help='The range of pool number for one device.'),
    cfg.StrOpt('fake_volume_range',
               default='1-2000',
               help='The range of volume number for one device.'),
    cfg.StrOpt('fake_api_time_range',
               default='0.1-0.5',
               help='The range of time cost for each API.'),
    cfg.StrOpt('fake_page_query_limit',
               default='500',
               help='The limitation of volumes for each query.'),
]

CONF.register_opts(fake_opts, "fake_driver")


LOG = log.getLogger(__name__)


MIN_WAIT, MAX_WAIT = 0.1, 0.5
MIN_POOL, MAX_POOL = 1, 100
MIN_VOLUME, MAX_VOLUME = 1, 2000
PAGE_LIMIT = 500


def get_range_val(range_str, t):
    try:
        rng = range_str.split('-')
        if len(rng) != 2:
            raise exception.InvalidInput
        min_val = t(rng[0])
        max_val = t(rng[1])
        return min_val, max_val
    except exception:
        LOG.error("Invalid range: {0}".format(range_str))
        raise exception.InvalidInput


def wait_random(low, high):
    @decorator.decorator
    def _wait(f, *a, **k):
        rd = random.randint(0, 100)
        secs = low + (high - low) * rd / 100
        time.sleep(secs)
        return f(*a, **k)

    return _wait


class FakeStorageDriver(driver.StorageDriver):
    """FakeStorageDriver shows how to implement the StorageDriver,
    it also plays a role as faker to fake data for being tested by clients.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        global MIN_WAIT, MAX_WAIT, MIN_POOL, MAX_POOL, MIN_VOLUME, MAX_VOLUME
        global PAGE_LIMIT
        MIN_WAIT, MAX_WAIT = get_range_val(
            CONF.fake_driver.fake_api_time_range, float)
        MIN_POOL, MAX_POOL = get_range_val(
            CONF.fake_driver.fake_pool_range, int)
        MIN_VOLUME, MAX_VOLUME = get_range_val(
            CONF.fake_driver.fake_volume_range, int)
        PAGE_LIMIT = int(CONF.fake_driver.fake_page_query_limit)

    def reset_connection(self, context, **kwargs):
        pass

    @wait_random(MIN_WAIT, MAX_WAIT)
    def get_storage(self, context):
        # Do something here
        sn = six.text_type(uuidutils.generate_uuid())
        total, used, free = self._get_random_capacity()
        raw = random.randint(2000, 3000)
        subscribed = random.randint(3000, 4000)
        return {
            'name': 'fake_driver',
            'description': 'fake driver.',
            'vendor': 'fake_vendor',
            'model': 'fake_model',
            'status': 'normal',
            'serial_number': sn,
            'firmware_version': '1.0.0',
            'location': 'HK',
            'total_capacity': total,
            'used_capacity': used,
            'free_capacity': free,
            'raw_capacity': raw,
            'subscribed_capacity': subscribed
        }

    @wait_random(MIN_WAIT, MAX_WAIT)
    def list_storage_pools(self, ctx):
        rd_pools_count = random.randint(MIN_POOL, MAX_POOL)
        LOG.info("###########fake_pools number for %s: %d" % (self.storage_id,
                                                              rd_pools_count))
        pool_list = []
        for idx in range(rd_pools_count):
            total, used, free = self._get_random_capacity()
            p = {
                "name": "fake_pool_" + str(idx),
                "storage_id": self.storage_id,
                "native_storage_pool_id": "fake_original_id_" + str(idx),
                "description": "Fake Pool",
                "status": "normal",
                "total_capacity": total,
                "used_capacity": used,
                "free_capacity": free,
            }
            pool_list.append(p)
        return pool_list

    def list_volumes(self, ctx):
        # Get a random number as the volume count.
        # rd_volumes_count = random.randint(MIN_VOLUME, MAX_VOLUME)
        rd_volumes_count = random.randint(MIN_VOLUME, MAX_VOLUME)
        LOG.info("###########fake_volumes number for %s: %d" % (
            self.storage_id, rd_volumes_count))
        loops = math.ceil(rd_volumes_count / PAGE_LIMIT)
        volume_list = []
        for idx in range(loops):
            start = idx * PAGE_LIMIT
            end = (idx + 1) * PAGE_LIMIT
            if idx == (loops - 1):
                end = rd_volumes_count
            vs = self._get_volume_range(start, end)
            volume_list = volume_list + vs
        return volume_list

    def add_trap_config(self, context, trap_config):
        pass

    def remove_trap_config(self, context, trap_config):
        pass

    def parse_alert(self, context, alert):
        pass

    def clear_alert(self, context, alert):
        pass

    @wait_random(MIN_WAIT, MAX_WAIT)
    def _get_volume_range(self, start, end):
        volume_list = []

        for i in range(start, end):
            total, used, free = self._get_random_capacity()
            v = {
                "name": "fake_vol_" + str(i),
                "storage_id": self.storage_id,
                "description": "Fake Volume",
                "status": "normal",
                "native_volume_id": "fake_original_id_" + str(i),
                "wwn": "fake_wwn_" + str(i),
                "total_capacity": total,
                "used_capacity": used,
                "free_capacity": free,
            }
            volume_list.append(v)
        return volume_list

    def _get_random_capacity(self):
        total = random.randint(1000, 2000)
        used = int(random.randint(0, 100) * total / 100)
        free = total - used
        return total, used, free



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pysnmp.carrier.asyncore.dispatch import AsyncoreDispatcher

from delfin import exception
from delfin.common import constants


def fake_storage_info():
    return {
        'id': 'abcd-1234-56789',
        'name': 'storage1',
        'vendor': 'fake vendor',
        'model': 'fake model',
        'serial_number': 'serial-1234',
    }


def fake_alert_model():
    return {'alert_id': '1050',
            'alert_name': 'SAMPLE_ALERT_NAME',
            'severity': constants.Severity.WARNING,
            'category': constants.Category.NOT_SPECIFIED,
            'type': constants.EventType.EQUIPMENT_ALARM,
            'sequence_number': 79,
            'description': 'Diagnostic event trace triggered.',
            'recovery_advice': 'NA',
            'resource_type': constants.DEFAULT_RESOURCE_TYPE,
            'location': 'Array id=000192601409,Component type=location1 '
                        'Group,Component name=comp1,Event source=symmetrix',
            }


def fake_v3_alert_source():
    return {'storage_id': 'abcd-1234-5678',
            'version': 'snmpv3',
            'engine_id': '800000d30300000e112245',
            'username': 'test1',
            'auth_key': 'YWJjZDEyMzQ1Njc=',
            'auth_protocol': 'HMACMD5',
            'privacy_key': 'YWJjZDEyMzQ1Njc=',
            'privacy_protocol': 'DES'
            }


def fake_v3_alert_source_list_with_one():
    return [
        {'storage_id': 'abcd-1234-5678',
         'version': 'snmpv3',
         'engine_id': '800000d30300000e112245',
         'username': 'test1',
         'auth_key': 'YWJjZDEyMzQ1Njc=',
         'auth_protocol': 'HMACMD5',
         'privacy_key': 'YWJjZDEyMzQ1Njc=',
         'privacy_protocol': 'DES'
         }
    ]


def null_alert_source_list():
    return []


def fake_v3_alert_source_list():
    return [
        {'storage_id': 'abcd-1234-5678',
         'version': 'snmpv3',
         'engine_id': '800000d30300000e112245',
         'username': 'test1',
         'auth_key': 'YWJjZDEyMzQ1Njc=',
         'auth_protocol': 'HMACMD5',
         'privacy_key': 'YWJjZDEyMzQ1Njc=',
         'privacy_protocol': 'DES'
         },
        {'storage_id': 'abcd-1234-5677',
         'version': 'snmpv3',
         'engine_id': '800000d30300000e112246',
         'username': 'test2',
         'auth_key': 'YWJjZDEyMzQ1Njc=',
         'auth_protocol': 'HMACMD5',
         'privacy_key': 'YWJjZDEyMzQ1Njc=',
         'privacy_protocol': 'DES'
         }
    ]


def parse_alert_exception():
    raise exception.InvalidResults("parse alert failed.")


def load_config_exception(para):
    raise exception.InvalidResults("load config failed.")


def mock_add_transport(snmpEngine, transportDomain, transport):
    snmpEngine.transportDispatcher = AsyncoreDispatcher()



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from delfin import exception
from delfin.drivers.hpe.hpe_3par import consts
from delfin.i18n import _

LOG = logging.getLogger(__name__)


class RestHandler(object):
    """Common class for Hpe 3parStor storage system."""
    # https://<storage_system>:8080/api/v1/credentials
    REST_AUTH_URL = '/api/v1/credentials'  # POST
    REST_LOGOUT_URL = '/api/v1/credentials/'  # DELETE <session key>
    REST_STORAGE_URL = '/api/v1/system'  # GET
    # GET  /api/v1/spacereporter  body cpg:cpgname
    REST_CAPACITY_URL = '/api/v1/capacity'
    REST_POOLS_URL = '/api/v1/cpgs'  # GET
    REST_VOLUMES_URL = '/api/v1/volumes'  # GET
    # category==LIFECYCLE 1  ALERT 2
    # GET    /api/v1/eventlog
    REST_ALERTS_URL = '/api/v1/eventlog?query="category EQ 2"'
    # X-HP3PAR-WSAPI-SessionKey
    REST_AUTH_KEY = 'X-HP3PAR-WSAPI-SessionKey'

    def __init__(self, restclient):
        self.restclient = restclient

    def call(self, url, data=None, method=None):
        """Send requests to server.
        If fail, try another RestURL.
        Increase the judgment of token invalidation
        """

        if self.restclient.session is None:
            self.restclient.init_http_head()
            if self.restclient.rest_auth_token is not None:
                # set taken in the header，key is X-HP3PAR-WSAPI-SessionKey
                self.restclient.session.headers[RestHandler.REST_AUTH_KEY] = \
                    self.restclient.rest_auth_token

        res = self.restclient.do_call(url, data, method,
                                      calltimeout=consts.SOCKET_TIMEOUT)
        # Judge whether the access failure is caused by
        # the token invalidation.
        # If the token fails, it will be retrieved again,
        # and the token will be accessed again
        if res is not None:
            # 403  The client request has an invalid session key.
            #      The request came from a different IP address
            # 409  Session key is being used.
            if (res.status_code == consts.ERROR_SESSION_INVALID_CODE
                    or res.status_code ==
                    consts.ERROR_SESSION_IS_BEING_USED_CODE):
                LOG.error(
                    "Failed to get token=={0}=={1}".format(res.status_code,
                                                           res.text))
                LOG.error("Failed to get token，relogin，Get token again！！！")
                # delete REST_AUTH_TOKEN，Recapture token
                try:
                    self.logout()
                except Exception as err:
                    LOG.error('logout error:{}'.format(err))

                self.restclient.rest_auth_token = None
                access_session = self.login()
                # if get token，Revisit url
                if access_session is not None:
                    res = self.restclient. \
                        do_call(url, data, method,
                                calltimeout=consts.SOCKET_TIMEOUT)
        else:
            LOG.error('login res is None')
        return res

    def get_resinfo_call(self, url, data=None, method=None):
        rejson = None
        # visit url
        res = self.call(url, data, method)
        if res is not None:
            if res.status_code == consts.SUCCESS_STATUS_CODES:
                rejson = res.json()
        return rejson

    def login(self):
        """Login Hpe3par storage array."""
        access_session = self.restclient.rest_auth_token

        if access_session is None:
            if self.restclient.san_address:
                url = RestHandler.REST_AUTH_URL

                data = {"user": self.restclient.rest_username,
                        "password": self.restclient.rest_password
                        }
                self.restclient.init_http_head()
                res = self.restclient. \
                    do_call(url, data, 'POST',
                            calltimeout=consts.SOCKET_TIMEOUT)

                if res is not None:
                    # check login status 201
                    if res.status_code == consts.LOGIN_SUCCESS_STATUS_CODES:
                        result = res.json()

                        access_session = result.get('key')
                        self.restclient.rest_auth_token = access_session
                        # set taken in the header，
                        # key is X-HP3PAR-WSAPI-SessionKey
                        self.restclient.session.headers[
                            RestHandler.REST_AUTH_KEY] = access_session
                    else:
                        LOG.error("Login error. URL: %(url)s\n"
                                  "Reason: %(reason)s.",
                                  {"url": url, "reason": res.text})
                        if 'invalid username or password' in res.text:
                            raise exception.InvalidUsernameOrPassword()
                        else:
                            raise exception.Invalid()
                else:
                    LOG.error('login res is None')
                    raise exception.InvalidResults('res is None')
            else:
                LOG.error('login Parameter error')
        else:
            LOG.error(
                "No login required！self.access_session have value=={}".format(
                    access_session))

        if access_session is None:
            msg = _("Failed to login with all rest URLs.")
            LOG.error(msg)
            raise exception.BadRequest(reason=msg)
        return access_session

    def logout(self):
        """Logout the session."""
        try:
            url = RestHandler.REST_LOGOUT_URL
            if self.restclient.rest_auth_token is not None:
                url = url + self.restclient.rest_auth_token
            self.restclient.rest_auth_token = None
            if self.restclient.san_address:
                self.call(url, method='DELETE')
        except Exception as err:
            LOG.error('logout error:{}'.format(err))
            raise exception.StorageBackendException(
                reason='Failed to Logout from restful')

    def get_storage(self):
        rejson = self.get_resinfo_call(RestHandler.REST_STORAGE_URL,
                                       method='GET')
        return rejson

    def get_capacity(self):
        rejson = self.get_resinfo_call(RestHandler.REST_CAPACITY_URL,
                                       method='GET')
        return rejson

    def get_all_pools(self):
        rejson = self.get_resinfo_call(RestHandler.REST_POOLS_URL,
                                       method='GET')
        return rejson

    def get_all_volumes(self):
        rejson = self.get_resinfo_call(RestHandler.REST_VOLUMES_URL,
                                       method='GET')
        return rejson
